#!/usr/bin/env python
"""
"""
import unittest
import nose
import subprocess

def test_CommandLineArguments():
    """
    Try to invoke different scripts of this pipeline, with the wrong arguments.
    Check that they always produce an error.
    """
    command_lines = [
        ("python", "src/get_gene_coords.py"),
        ("python", "src/get_gene_coords.py", "-f", "notexistingfile.txt"),
        ("python", "src/get_gene_coords.py", "--wrongoption")


    ]
    for params_set in command_lines:
        yield check_BadCommandLine, params_set


@nose.tools.raises(subprocess.CalledProcessError)
def check_BadCommandLine(params_set):
    out = subprocess.check_output(params_set,stderr=subprocess.STDOUT)

if __name__ == '__main__':
    import logging
    import doctest
    logging.basicConfig(level=logging.DEBUG)
    doctest.testmod()





#!/usr/bin/env python
"""
Get the genotypes of the genes listed in <gene_list> from 1000genomes.

Uses tabix to retrieve the vcf file.

Usage
=========

   $: python get_genotypes.py -l data/n-glycan.coords


Inputs
==========

the List of genes file must contain the coordinates of each gene in the analysis, in the following format:

::
    
    DOLK    N-glycan        substrates      chr9    131707808       131710012
    RPN1    N-glycan        ost_complex     chr3    128338812       128399918
    ALG8    N-glycan        precursor_biosynthesis  chr11   77811987        77850699
    ALG9    N-glycan        precursor_biosynthesis  chr11   111652918       111750181
    UAP1    N-glycan        substrates      chr1    162531295       162569633
    ST6GAL1 N-glycan        branching2      chr3    186648314       186796341
    ALG2    N-glycan        precursor_biosynthesis  chr9    101978706       101984246
    ALG3    N-glycan        precursor_biosynthesis  chr3    183960116       183967313
    ALG1    N-glycan        precursor_biosynthesis  chr16   5083702 5137380
    ALG6    N-glycan        precursor_biosynthesis  chr1    63833260        63904233
    GANAB   N-glycan        cnx_crt chr11   62392297        62414104

Note that SNPs in the 1000genomes project are encoded to hg19, so the coordinates must be all refSeq hg19. You can use the script get_gene_coords.py to retrieve the coordinates automatically.


"""
import optparse
import subprocess
import os
import glob


def get_options():
    parser = optparse.OptionParser(usage="python %prog -l genes_coords.txt", description=__doc__)
    parser.add_option('-g', '-c', '-f', '--coordinates_file', '--coordinates', '--genes', dest='genes', 
            help='File containing the coordinates of each gene. The first column must contain the gene name; \
                        the second and third columns may contain anything, usually a pathway and subpathway classification;\
                        the third, fourth, and fifth column must contain the chromosome, start and end positions.\
                        See the documentation in the code of this script for more help. Default: %default', default=False)
    parser.add_option('-o', '--outputpath', '--output_path', dest='outputpath', 
            help='Directory where the vcf files will be saved. It must be already existing, because the script won\'t create it automatically. Default: %default', 
            default="./data/vcf/")
    parser.add_option('-u', '--url', '--baseurl', dest='baseurl', 
            help='Base URL to the file containing the whole genome data. Use {0} to replace the chromosome number. Default: %default', 
            default = "ftp://ftp.1000genomes.ebi.ac.uk/vol1/ftp/phase1/analysis_results/integrated_call_sets/ALL.chr{0}.integrated_phase1_v3.20101123.snps_indels_svs.genotypes.vcf.gz")
    parser.add_option('-r', '--flank', dest='flank', 
            help='Include a flanking region of size n, upstream and downstream of the gene start and end. Default: %default', default=0)
    parser.add_option('-i', '--index', '--force-generate-index', dest='force_index', action='store_true',
            help='Force generation of tabix index file. Default: %default', default=False)
    parser.add_option('-d', '--download', '--force-download', dest='force_download', action='store_true',
            help='Force download of vcf files. Default: %default', default=False)
    parser.add_option('--get_individuals_index', dest='get_individuals', action='store_true',
            help='Force download of Individuals index. Default: %default', default=False)
    parser.add_option('-p', '--only-print', dest='onlyprint', action='store_true',
            help='Only print commands, without actually downloading. Default: %default', default=False)
    parser.add_option('--debug', dest='debug', action='store_true', default=True)
    (options, args) = parser.parse_args()
    
    if options.debug is True:
        logging.basicConfig(level=logging.DEBUG, filename='logs/get_genotypes.log')

    if (options.genes is False) and (len(args) == 0):
        parser.print_help()
        parser.error('get_genotypes.py: genes file not defined.')

    try:
        genes_path = ''
        if options.genes is not False:
            genes_path = options.genes
        elif args != '':
            genes_path = args[0]
        if os.stat(genes_path).st_size == 0:
            parser.error("file {0} is empty".format(genes_path))
    except:
        print __doc__
        parser.error("Can not open genes file")

    genes = read_genes_file(genes_path)
    logging.debug(genes[:2])

    return (genes, options)

def read_genes_file(genes_path):
    """
    DOLK    N-glycan        substrates      chr9    131707808       131710012
    RPN1    N-glycan        ost_complex     chr3    128338812       128399918
    ALG8    N-glycan        precursor_biosynthesis  chr11   77811987        77850699
    ALG9    N-glycan        precursor_biosynthesis  chr11   111652918       111750181
    UAP1    N-glycan        substrates      chr1    162531295       162569633
    ST6GAL1 N-glycan        branching2      chr3    186648314       186796341
    ALG2    N-glycan        precursor_biosynthesis  chr9    101978706       101984246
    ALG3    N-glycan        precursor_biosynthesis  chr3    183960116       183967313
    ALG1    N-glycan        precursor_biosynthesis  chr16   5083702 5137380
    ALG6    N-glycan        precursor_biosynthesis  chr1    63833260        63904233
    GANAB   N-glycan        cnx_crt chr11   62392297        62414104

    """
    genes = [line.split() for line in open(genes_path, 'r')]
    return genes


def get_genotypes(genes, baseurl, force_index=False, force_download=False, get_individuals=False, flank=0, outputpath='data/vcf/', print_only=False):
    """
    Call tabix on 1000genomes server to get coordinates.

    Note: the URL to 1000genomes may change over time. If something doesn't work, check it.
    """
##    tabix_1000genomes_url = "ftp://ftp.1000genomes.ebi.ac.uk/vol1/ftp/release/20100804/ALL.2of4intersection.20100804.genotypes.vcf.gz"
##    tabix_1000genomes_url = "ftp://ftp.1000genomes.ebi.ac.uk/vol1/ftp/release/20110521/ALL.chr{0}.phase1_release_v3.20101123.snps_indels_svs.genotypes.vcf.gz"
##                             ftp://ftp.1000genomes.ebi.ac.uk/vol1/ftp/release/20110521 
#    tabix_1000genomes_url = "ftp://ftp.1000genomes.ebi.ac.uk/vol1/ftp/phase1/analysis_results/integrated_call_sets/ALL.chr{0}.integrated_phase1_v3.20101123.snps_indels_svs.genotypes.vcf.gz"
    if baseurl[0:3] not in ("ftp", "htt"):
        tabix_1000genomes_url = '../../' + baseurl
        tabix_indexfile_template = baseurl + '.tbi'
    else:
        tabix_1000genomes_url = baseurl
        tabix_indexfile_template = './data/tabix_indexes/' + baseurl.split('/')[-1] + '.tbi'
    individuals_file = "phase1_integrated_calls.20101123.ALL.panel"
#    ftp://ftp.1000genomes.ebi.ac.uk/vol1/ftp/release/20110521/phase1_integrated_calls.20101123.ALL.panel
    individuals_url = tabix_1000genomes_url.rsplit('/', 1)[0] + '/' + individuals_file

    print flank
    flank = int(flank)


    # Get individuals' info
    if get_individuals is True:
        print "\nDownloading individual infos\n"
        print("wget {0}".format(individuals_url))
        subprocess.call("wget {0}".format(individuals_url).split())
        print("mv {0} data/individuals/{1}.individuals".format(individuals_file, individuals_file))
        subprocess.call("mv {0} data/individuals/{1}.individuals".format(individuals_file, individuals_file).split())

#    report_table_content = 'gene\tpathway\tsubpathway\tchromosome\tstart\tend\ttot_snps\tgene_length\tsnp_per_kb\n'
    report_table_content = 'gene\tpathway\tsubpathway\tchromosome\tstart\tend\tgene_length\n'
    for fields in genes:
        (gene,pathway,subpathway,chromosome,start,end) = fields[:6]
        start = int(start) - flank
        end   = int(end)   + flank
        
        chromosome = chromosome.replace('chr', '')
        print "\nDownloading genotypes for gene {0}, {1}:{2}-{3}".format(gene, chromosome, start,end)
        index_command = "../../bin/tabix -f -p vcf {0}".format(tabix_1000genomes_url.format(chromosome))
        command = "../../bin/tabix -p vcf -h {0} {1}:{2}-{3}".format(tabix_1000genomes_url.format(chromosome), chromosome, start, end)
        print (tabix_indexfile_template.format(chromosome))
        if force_index is True or not (os.path.exists(tabix_indexfile_template.format(chromosome))):
            print "- generating indexes", index_command
            if print_only is False:
                subprocess.call(index_command.split(), cwd='./data/tabix_indexes/')
        outputfilepath = outputpath + '/' + gene + '.vcf'
        gzoutputfilepath = outputfilepath + '.gz'

        if (os.path.isfile(gzoutputfilepath)) and (os.stat(gzoutputfilepath).st_size > 0) and (force_download is not True):
            print "- skipping download, as genotypes for gene {0} are already available in data/vcf/{1}.vcf. Use --force-download option to download again.".format(gene, gene)
        else:
            print "- executing command: " + command
            if print_only is False:
                outputfile = open(outputfilepath, 'w')
                subprocess.call(command.split(), cwd='./data/tabix_indexes/', stdout=outputfile)
                subprocess.call(["gzip", "-f", outputfile.name])

        # Calculate how many SNPs correspond to the gene, and update Table S1
        gene_length = int(end) - int(start)
        report_table_content += "{0}\t{1}\n".format('\t'.join(fields), gene_length)
        

    return report_table_content

def main():
    (genes, options) = get_options()
    report_table_content = get_genotypes(genes, options.baseurl, options.force_index, options.force_download, options.get_individuals, options.flank, options.outputpath, options.onlyprint)
    print report_table_content
    pathway = options.genes.split('/')[-1].rsplit('.')[0]
#    report_outputfilepath = options.report_outputpath
#    report_outputfile = open(report_outputfilepath, 'w')
#    report_outputfile.write(report_table_content)


if __name__ == '__main__':
    import logging
    import doctest
    logging.basicConfig(level=logging.DEBUG)
    doctest.testmod()
    main()



#!/usr/bin/env python
import subprocess
import nose
import numpy
import operator
import random

@nose.plugins.attrib.attr('slow')
def testValuesByNodeSimulationsReport():
    """
    >>> command = "python src/get_simulations_background.py -t values_by_node -f test/test_data/simulations/binary/globalpops/"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)
    >>> output = output.splitlines()

    >>> print output[0]
    genotype name window whole_gene_nsnps network_gene network_continent network_distance_definition cont_str n_datapoints degree closeness betweenness

    >>> output.sort()

    #NOTE: the blankline at the end is generated by subprocess.check_output, not by the script
    >>> for line in output[:5]:
    ...     print line #doctest: +NORMALIZE_WHITESPACE 
    0001111111  neutral_sim window_35   360 neutral_sim glob    1   ASN 21  0   0.0323  0.0
    0001111111  neutral_sim_ASN window_35   360 neutral_sim ASN 1   ASN 21  0   0.0833  0.0
    0010011111  neutral_sim window_35   360 neutral_sim glob    1   ASN 181 2   0.0932  22.0
    0010011111  neutral_sim_ASN window_35   360 neutral_sim ASN 1   ASN 181 1   0.0909  0.0
    0010101111  neutral_sim window_35   360 neutral_sim glob    1   ASN 1   1   0.0333  0.0


    """

def testDebugSimulationsBackgroundResampling():
    """
    Check that the individuals subsampled are always the same.

    >>> command = "python src/get_simulations_background.py  -f ./test/test_data/simulations/binary/globalpops_allpops/ --random_seed 2 -n 50 -t individuals_line"

    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)

    #NOTE: the blankline at the end is generated by subprocess.check_output, not by the script
    >>> output_lines = output.split('\\n')

    >>> for line in output_lines: #doctest: +NORMALIZE_WHITESPACE 
    ...     print line[0:100]
    <BLANKLINE>
    -----neutral_sim EUR  hap0_pop1 EUR pop1  hap1_pop1 EUR pop1  hap2_pop1 EUR pop1  hap3_pop1 EUR pop1
    -----neutral_sim EUR  hap0_pop1 EUR pop1  hap1_pop1 EUR pop1  hap2_pop1 EUR pop1  hap3_pop1 EUR pop1
    -----neutral_sim EUR  hap0_pop1 EUR pop1  hap1_pop1 EUR pop1  hap2_pop1 EUR pop1  hap3_pop1 EUR pop1
    <BLANKLINE>

    >>> output_lines = output_lines[1:-1]
    >>> output_fields = [line.split() for line in output_lines]

    # Check that all the columns, except for the ones starting with "--", have the same values for all the output lines.
    # This means that all the selected individuals are the same, for each line.

    # Select 500 random columns:
    >>> min_len = min(len(output_fields[0]), len(output_fields[1]), len(output_fields[2]))
    >>> random_columns = random.sample(xrange(min_len), 500)
    >>> fields_getter = operator.itemgetter(*random_columns)
    >>> sim1 = fields_getter(output_fields[0])
    >>> sim2 = fields_getter(output_fields[1])
    >>> sim3 = fields_getter(output_fields[2])
    >>> for column_id in xrange(len(sim1)):
    ...     if not sim1[column_id].startswith("----"):
    ...        if not sim1[column_id] == sim2[column_id] == sim3[column_id]:
    ...             print "not matching values", sim1[column_id], sim2[column_id], sim3[column_id]
    """


def testValuesByGeneSimulationsReport():
    """
    >>> command = "python src/get_simulations_background.py -f test/test_data/simulations/binary/globalpops/"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)
    >>> output = output.splitlines()
    >>> print output[0]
    gene continent distance_definition n_snps central_snp chromosome central_snp_position upstream_position downstream_position region_size distance_from_upstream_margin distance_from_downstream_margin n_datapoints av_datapoints_per_node n_vertices n_edges n_components av_path_length var_path_length diameter av_w_path_length av_w_path_length_inv av_degree var_degree median_degree max_degree av_w_degree av_w_degree_inv av_closeness var_closeness median_closeness max_closeness density av_betweenness var_betweenness median_betweenness max_betweenness window whole_gene_nsnps
    >>> output = output[1:]
    >>> output.sort(key=lambda x:x[-3:])

    >>> outputvalues = [v.split() for v in output]
    >>> [(v[13]) for v in outputvalues] == ['58.7742', '68.9091', '47.6667', '41.0', '15.6522', '15.0', '12.0', '10.0', '8.8889', '6.0', '17.1429']
    True

    >>> mean_av_datapoints = numpy.mean([float(v[13]) for v in outputvalues])
    >>> print round(mean_av_datapoints, 4)
    27.3667

    >>> for line in output:
    ...     print line  #doctest: +NORMALIZE_WHITESPACE
    neutral_sim glob 1 10 NA NA nan nan nan nan nan nan 1822 58.7742 31 30 6 3.7374 3.2124 9 0.4461 63.1098 1.9355 2.1249 2.0 6 0.1043 721.1816 0.0842 0.0008 0.098 0.1095 0.0645 24.5484 1928.5269 6.0 155.1667 window_35 360
    neutral_sim EUR 1 10 NA NA nan nan nan nan nan nan 758 68.9091 11 9 2 1.6667 0.2308 2 0.7175 3.921 1.6364 2.2314 1.0 6 0.3411 105.8128 0.162 0.0009 0.1818 0.2 0.1636 1.6364 18.595 0.0 15.0 window_35 360
    neutral_sim ASN 1 10 NA NA nan nan nan nan nan nan 572 47.6667 12 5 7 1.1667 0.1667 2 0.1809 13.7025 0.8333 0.3056 1.0 2 0.0775 70.2283 0.0911 0.0 0.0909 0.1 0.0758 0.0833 0.0764 0.0 1.0 window_35 360
    neutral_sim AFR 1 10 NA NA nan nan nan nan nan nan 492 41.0 12 10 3 2.7838 2.5075 6 0.7313 32.4049 1.6667 0.7222 2.0 3 0.1651 232.1307 0.1635 0.002 0.1792 0.2075 0.1515 5.5 45.4583 1.75 16.0 window_35 360
    neutral_sim glob 1 10 NA NA nan nan nan nan nan nan 360 15.6522 23 20 6 2.935 2.0121 7 0.4047 41.6849 1.7391 2.1059 2.0 7 0.1104 104.895 0.0884 0.0008 0.1038 0.117 0.0791 10.3478 311.6616 1.0 84.0 window_35 361
    neutral_sim EUR 1 10 NA NA nan nan nan nan nan nan 120 15.0 8 4 4 1.8 0.6222 3 0.4667 9.2108 1.0 1.0 1.0 3 0.1323 14.0625 0.1874 0.0024 0.2154 0.2414 0.1429 1.0 3.25 0.0 5.0 window_35 361
    neutral_sim ASN 1 10 NA NA nan nan nan nan nan nan 120 12.0 10 7 3 2.0 1.0588 4 0.3551 16.6272 1.4 0.64 1.0 3 0.1292 38.7697 0.1539 0.0009 0.1699 0.1875 0.1556 1.8 6.96 0.0 7.0 window_35 361
    neutral_sim AFR 1 10 NA NA nan nan nan nan nan nan 120 10.0 12 12 1 2.8182 1.9049 6 0.5909 18.9151 2.0 1.3333 2.0 5 0.2194 62.9627 0.3739 0.0073 0.3798 0.55 0.1818 10.0 137.1667 10.0 41.5 window_35 361
    neutral_sim glob 1 10 NA NA nan nan nan nan nan nan 240 8.8889 27 21 8 2.7582 1.8965 6 0.1165 99.4246 1.5556 0.9877 1.0 4 0.0481 141.2267 0.051 0.0001 0.0433 0.0645 0.0598 5.9259 99.013 0.0 30.5 window_37 383
    neutral_sim EUR 1 10 NA NA nan nan nan nan nan nan 120 6.0 20 18 4 2.8182 1.8516 6 0.2301 51.2801 1.8 1.06 2.0 4 0.0825 89.0 0.0911 0.0006 0.1033 0.1152 0.0947 8.0 117.075 3.0 30.5 window_37 383
    neutral_sim ASN 1 10 NA NA nan nan nan nan nan nan 120 17.1429 7 3 4 1.0 0.0 1 0.3167 3.7221 0.8571 0.1224 1.0 1 0.1357 18.08 0.1633 0.0001 0.1667 0.1667 0.1429 0.0 0.0 0.0 0.0 window_37 383


    """

@nose.plugins.attrib.attr('slow')
def testValuesByGeneSimulationsBigWindowReport():
    """
    >>> command = "python src/get_simulations_background.py -f test/test_data/simulations/binary/globalpops/ -w 30"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)
    >>> print output[:1000] #doctest: +NORMALIZE_WHITESPACE
    gene continent distance_definition n_snps central_snp chromosome central_snp_position upstream_position downstream_position region_size distance_from_upstream_margin distance_from_downstream_margin n_datapoints av_datapoints_per_node n_vertices n_edges n_components av_path_length var_path_length diameter av_w_path_length av_w_path_length_inv av_degree var_degree median_degree max_degree av_w_degree av_w_degree_inv av_closeness var_closeness median_closeness max_closeness density av_betweenness var_betweenness median_betweenness max_betweenness window whole_gene_nsnps
    neutral_sim glob 1 30 NA NA nan nan nan nan nan nan 360 4.9315 73 20 53 1.9535 1.2359 5 0.0408 128.0379 0.5479 0.5491 0.0 4 0.0076 95.3552 0.0139 0.0 0.0137 0.0151 0.0076 0.5616 6.027 0.0 17.0 window_11 371
    neutral_sim EUR 1 30 NA NA nan nan nan nan nan nan 120 8.5714 14 3 11 1.0 0.0 1 0.1806 9.972 0.4286 0.2449 0.0 1 0.0387 20.5179 0.0738 0.0 0.0714 0.0769 0.033 0.0 0.0 0.0 0.0 window_11 371
    neutral_sim ASN 1 30 NA NA nan

    """
def testDebugReportEvenWindowIndividuals():
    """
    >>> command = "python src/get_simulations_background.py -f test/test_data/simulations/binary/globalpops/ -w 6 -t individuals "
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)
    >>> print output[:200] #doctest: +NORMALIZE_WHITESPACE
    <BLANKLINE>
    -----
    neutral_sim EUR
    hap1_pop1 EUR pop1
    hap2_pop1 EUR pop1
    hap3_pop1 EUR pop1
    hap4_pop1 EUR pop1
    hap5_pop1 EUR pop1
    hap6_pop1 EUR pop1
    hap7_pop1 EUR pop1
    hap8_pop1 EUR pop1
    hap9_pop1 EUR pop1
    hap10_

    """

def testDebugReportEvenWindow():
    """
    >>> command = "python src/get_simulations_background.py -f test/test_data/simulations/binary/globalpops/ -w 6"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)
    >>> print output[:1000] #doctest: +NORMALIZE_WHITESPACE
    gene continent distance_definition n_snps central_snp chromosome central_snp_position upstream_position downstream_position region_size distance_from_upstream_margin distance_from_downstream_margin n_datapoints av_datapoints_per_node n_vertices n_edges n_components av_path_length var_path_length diameter av_w_path_length av_w_path_length_inv av_degree var_degree median_degree max_degree av_w_degree av_w_degree_inv av_closeness var_closeness median_closeness max_closeness density av_betweenness var_betweenness median_betweenness max_betweenness window whole_gene_nsnps
    neutral_sim glob 1 6 NA NA nan nan nan nan nan nan 360 30.0 12 10 4 2.1111 0.7873 4 0.4461 12.7717 1.6667 1.8889 1.5 4 0.2139 34.6374 0.1774 0.0031 0.2019 0.2292 0.1515 3.3333 23.9815 0.6667 14.0 window_58 359
    neutral_sim EUR 1 6 NA NA nan nan nan nan nan nan 120 20.0 6 4 2 1.6 0.2667 2 0.4333 9.1389 1.3333 1.5556 1.0 4 0.1806 17.4565 0.3675 0.0098 0.3846 0.5 0.2667 1.0 5.0 0.0 6.0 window_58 359
    neutral_sim ASN 1 6 NA NA n


    """
def testDebugReportOddWindow():
    """

    >>> import glob
    >>> for testfile in sorted(glob.glob("test/test_data/simulations/binary/globalpops/*")): print testfile
    test/test_data/simulations/binary/globalpops/bestfit_neutral_replica_1.hap-0.binary.gz
    test/test_data/simulations/binary/globalpops/bestfit_neutral_replica_10.hap-0.binary.gz
    test/test_data/simulations/binary/globalpops/bestfit_neutral_replica_2.hap-0.binary.gz

    >>> command = "python src/get_simulations_background.py -f test/test_data/simulations/binary/globalpops/ -w 5"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)
    >>> print output[:1000] #doctest: +NORMALIZE_WHITESPACE
    gene continent distance_definition n_snps central_snp chromosome central_snp_position upstream_position downstream_position region_size distance_from_upstream_margin distance_from_downstream_margin n_datapoints av_datapoints_per_node n_vertices n_edges n_components av_path_length var_path_length diameter av_w_path_length av_w_path_length_inv av_degree var_degree median_degree max_degree av_w_degree av_w_degree_inv av_closeness var_closeness median_closeness max_closeness density av_betweenness var_betweenness median_betweenness max_betweenness window whole_gene_nsnps
    neutral_sim glob 1 5 NA NA nan nan nan nan nan nan 360 40.0 9 6 3 2.1905 1.0619 4 0.9257 7.8792 1.3333 1.3333 1.0 4 0.283 15.3591 0.2261 0.0042 0.25 0.2963 0.1667 2.7778 18.1728 0.0 12.0 window_70 359
    neutral_sim EUR 1 5 NA NA nan nan nan nan nan nan 120 24.0 5 3 2 1.5 0.3 2 1.0833 2.0809 1.2 0.96 1.0 3 0.4333 9.5351 0.38 0.0096 0.4 0.5 0.3 0.6 1.44 0.0 3.0 window_70 359
    neutral_sim ASN 1 5 NA NA nan nan nan nan nan nan 12

    """
#@nose.plugins.attrib.attr('slow')
def testValuesByNodeSimulationsReportDifferentNIndividuals():
    """

    NOTE: this test may fail in some systems. I believe it depends on the random_seed, and on the version of the libraries installed.

    >>> command = "python src/get_simulations_background.py -t values_by_gene -f test/test_data/simulations/binary/globalpops/ -i 30 --random_seed 2" 
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)

    >>> output = output.splitlines()
    >>> print output[0]
    gene continent distance_definition n_snps central_snp chromosome central_snp_position upstream_position downstream_position region_size distance_from_upstream_margin distance_from_downstream_margin n_datapoints av_datapoints_per_node n_vertices n_edges n_components av_path_length var_path_length diameter av_w_path_length av_w_path_length_inv av_degree var_degree median_degree max_degree av_w_degree av_w_degree_inv av_closeness var_closeness median_closeness max_closeness density av_betweenness var_betweenness median_betweenness max_betweenness window whole_gene_nsnps

    >>> output = output[1:]
    >>> output.sort(key=lambda x:x[-3:])

    >>> outputvalues = [v.split() for v in output]
    >>> print sorted([(float(v[13])) for v in outputvalues])
    [2.0, 2.1429, 2.3077, 2.5, 3.0, 3.75, 4.2857, 4.2857, 7.5, 7.5, 8.8889, 10.0, 15.6522, 58.7742]


    NOTE: oldvalues in henikoff:
    [2.1429, 2.3077, 2.5, 2.7273, 3.0, 3.3333, 4.2857, 4.2857, 5.0, 5.0, 6.0, 8.8889, 15.6522, 58.7742]
    [1.875, 2.1429, 2.5, 3.0, 3.0, 3.3333, 4.2857, 4.2857, 5.0, 6.0, 6.0, 8.8889, 15.6522, 58.7742]
    NOTE: oldvalues in laptop:
    [10.0, 15.6522, 2.0, 2.5, 3.0, 3.0, 3.3333, 3.3333, 3.3333, 3.75, 5.0, 58.7742, 6.0, 8.8889]

    >>> mean_av_datapoints = numpy.mean([float(v[13]) for v in outputvalues])
    >>> print round(mean_av_datapoints, 4)
    9.4705

    NOTE: on henikoff, before fixing the order of lines in output (commit 1634:a4aeff4bd7d9)
    9.1832

    NOTE: on laptop:
    8.9099

    #NOTE: the blankline at the end is generated by subprocess.check_output, not by the script
    >>> for line in output[:5]:
    ...     print line  #doctest: +NORMALIZE_WHITESPACE 
    neutral_sim glob 1 10 NA NA nan nan nan nan nan nan 1822 58.7742 31 30 6 3.7374 3.2124 9 0.4461 63.1098 1.9355 2.1249 2.0 6 0.1043 721.1816 0.0842 0.0008 0.098 0.1095 0.0645 24.5484 1928.5269 6.0 155.1667 window_35 360
    neutral_sim global_sub 1 10 NA NA nan nan nan nan nan nan 30 2.3077 13 5 8 1.4444 0.2778 2 0.2333 12.3333 0.7692 0.7929 1.0 3 0.0692 14.8901 0.0868 0.0001 0.0902 0.1 0.0641 0.3077 0.6746 0.0 3.0 window_35 360
    neutral_sim EUR 1 10 NA NA nan nan nan nan nan nan 30 7.5 4 2 2 1.3333 0.3333 2 0.6222 2.8718 1.0 0.5 1.0 2 0.2333 11.25 0.4018 0.0085 0.4286 0.5 0.3333 0.25 0.1875 0.0 1.0 window_35 360
    neutral_sim ASN 1 10 NA NA nan nan nan nan nan nan 30 3.75 8 3 5 1.25 0.25 2 0.175 9.8214 0.75 0.4375 1.0 2 0.0583 12.8125 0.1441 0.0003 0.1429 0.1667 0.1071 0.125 0.1094 0.0 1.0 window_35 360
    neutral_sim AFR 1 10 NA NA nan nan nan nan nan nan 30 7.5 4 0 4 nan 0.0 0 nan nan 0.0 0.0 0.0 0 0.0 0.0 0.25 0.0 0.25 0.25 0.0 0.0 0.0 0.0 0.0 window_35 360


    """




#!/usr/bin/env python
"""
"""
#from test_SubgraphByPop import SimpleNetwork
import binary_to_network
import unittest
import nose
import subprocess
import logging
from src.GenotypeNetwork import GenotypeNetworkInitError
logging.basicConfig(level=logging.DEBUG)

class SimpleNetworkFromFile(unittest.TestCase):
    gene_name = 'ALG3'
    binary_genotypes_file = 'test/test_data/binary_genotypes/{0}.binary'.format(gene_name)
    individuals_file = 'test/test_data/phase1_integrated_calls.20101123.ALL.panel.individuals'

    expected_genotypes = ['0000000000000000', '1010000100000010', '0000000100000000', '1010000100000000', '0010101100001001', '0010000100000010', '0001000000000000', '0000000000000100', '1010010111010000', '0010000100100010', '1010010110010000', '1110010111010000', '0000000000000010', '0010000000000000', '0010000100000000', '0110010111010000', '0010010110010000', '0010010100100010', '1010000000000000', '1010010100100010', '1010000100000100', '1010000110010000']


    individuals_dict = binary_to_network.parse_individuals_file(individuals_file, pickle_from_file_if_possible=False) #TODO: de-activate pickling of this variable

    def setUp(self):
#        self.network = binary_to_network.generate_network(self.binary_genotypes_file, self.individuals_dict) 
        self.all_genotypes, self.individuals, self.individuals_dict, self.gene_name = binary_to_network.parse_binary_file(
                    self.binary_genotypes_file, self.individuals_dict)
        print self.binary_genotypes_file
        self.network = binary_to_network.generate_network(self.all_genotypes, self.individuals, self.individuals_dict, self.gene_name) 

    def test_Genotypes(self):
        print self.expected_genotypes
        print self.network.vs['genotype']
#        nose.tools.assert_sequence_equal(self.expected_genotypes, self.network.vs['genotype'])
        nose.tools.assert_equal(self.expected_genotypes, self.network.vs['genotype'])

    def test_Windows515(self):
        """test a window from position 5 to 15"""
        upstream = 5
        downstream = 15
#        self.network_window1 = binary_to_network.generate_network(self.binary_genotypes_file, self.individuals_dict, upstream=upstream, downstream=downstream) 
        logging.debug (("expected length:10 or less AAAAAAAAAAAAAAAA", self.all_genotypes[0:10]))
        self.network_window1 = binary_to_network.generate_network(self.all_genotypes, self.individuals, self.individuals_dict, self.gene_name, upstream=upstream, downstream=downstream) 
        expected_genotypes_window1 = set([g[upstream-1:downstream] for g in self.expected_genotypes])
        network_window1_genotypes = set(self.network_window1.vs['genotype'])
        nose.tools.assert_equal(expected_genotypes_window1.difference(network_window1_genotypes), set([]))

    def test_WindowsEndIsBiggerThanChromosomeLength(self):
        upstream = 5
        downstream = 1000
        self.network_window1 = binary_to_network.generate_network(self.all_genotypes, self.individuals, self.individuals_dict, self.gene_name, upstream=upstream, downstream=downstream) 
        expected_genotypes_window1 = set([g[upstream-1:len(self.expected_genotypes[0])] for g in self.expected_genotypes])
        network_window1_genotypes = set(self.network_window1.vs['genotype'])
        nose.tools.assert_equal(expected_genotypes_window1.difference(network_window1_genotypes), set([]))

    @nose.tools.raises(ValueError)
    def test_WindowsStartisBiggerThanEnd(self):
        upstream = 20
        downstream = 5
        self.network_window1 = binary_to_network.generate_network(self.all_genotypes, self.individuals, self.individuals_dict, self.gene_name, upstream=upstream, downstream=downstream) 

    def test_WindowsEndIsNone(self):
        upstream = 5
        downstream = None
        self.network_window1 = binary_to_network.generate_network(self.all_genotypes, self.individuals, self.individuals_dict, self.gene_name, upstream=upstream, downstream=downstream) 
        expected_genotypes_window1 = set([g[upstream-1:len(self.expected_genotypes[0])] for g in self.expected_genotypes])
        network_window1_genotypes = set(self.network_window1.vs['genotype'])
        nose.tools.assert_equal(expected_genotypes_window1.difference(network_window1_genotypes), set([]))

    def test_WindowsStartIsNone(self):
        upstream = None
        downstream = 15
        self.network_window1 = binary_to_network.generate_network(self.all_genotypes, self.individuals, self.individuals_dict, self.gene_name, upstream=upstream, downstream=downstream) 
        expected_genotypes_window1 = set([g[0:downstream] for g in self.expected_genotypes])
        network_window1_genotypes = set(self.network_window1.vs['genotype'])
        print expected_genotypes_window1
        print network_window1_genotypes
        nose.tools.assert_equal(expected_genotypes_window1.difference(network_window1_genotypes), set([]))

    def test_WindowsStartAndEndAreNone(self):
        upstream = None
        downstream = None
        self.network_window1 = binary_to_network.generate_network(self.all_genotypes, self.individuals, self.individuals_dict, self.gene_name, upstream=upstream, downstream=downstream) 
        expected_genotypes_window1 = set([g for g in self.expected_genotypes])
        network_window1_genotypes = set(self.network_window1.vs['genotype'])
        nose.tools.assert_equal(expected_genotypes_window1.difference(network_window1_genotypes), set([]))


class MOGS(SimpleNetworkFromFile):
    gene_name = 'MOGS'
    binary_genotypes_file = 'test/test_data/binary_genotypes/{0}.binary'.format(gene_name)
    individuals_file = 'test/test_data/phase1_integrated_calls.20101123.ALL.panel.individuals'
    expected_genotypes = ['0000000', '0000001', '0000100', '0011000', '0000010', '0011001', '0011100', '0111000', '0011010', '1010000', '0000110', '0000101', '0010000']
    individuals_dict = binary_to_network.parse_individuals_file(individuals_file, pickle_from_file_if_possible=False) #TODO: de-activate pickling of this variable


#@nose.tools.raises(src.GenotypeNetwork.GenotypeNetworkInitError)
#@nose.tools.raises(GenotypeNetworkInitError)
def test_ALG13_triallelicSNPs():
    """
    Test that creating a network from ALG13, which has only invalid genotypes, returns an Error

    ALG13 contains some "2" in the genotypes. 
    Unfortunately this file doesn't have any valid genotype.
    """
    gene_name = 'ALG13'
    binary_genotypes_file = 'test/test_data/binary_genotypes/{0}.binary'.format(gene_name)
    individuals_file = 'test/test_data/phase1_integrated_calls.20101123.ALL.panel.individuals'
    individuals_dict = binary_to_network.parse_individuals_file(individuals_file, pickle_from_file_if_possible=False) #TODO: de-activate pickling of this variable
    all_genotypes, individuals, individuals_dict, gene_name = binary_to_network.parse_binary_file(binary_genotypes_file, individuals_dict)
#    with nose.tools.raises(GenotypeNetworkInitError):
    with nose.tools.raises(src.GenotypeNetwork.GenotypeNetworkInitError):
        network = binary_to_network.generate_network(all_genotypes, individuals, individuals_dict, gene_name) 
#    network = binary_to_network.generate_network(binary_genotypes_file, individuals_dict) 


#def test_StartAndEndPositions():
#    """
#    generate networks by varying the upstream and downstream positions
#    """


#def check_BinaryToNetworkCommandLineInterface():
#    comms = ['python src/binary_to_network -f ']

       
if __name__ == '__main__':
    import logging
    import doctest
    logging.basicConfig(level=logging.DEBUG)
    doctest.testmod()





#!/usr/bin/env python
import subprocess

def testDebugReport():
    """
    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/ALG10B.genes -p test/test_data/params/debug.yaml --no-debug --data_basedir=./test/test_data -w 50 -t values_by_node"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)

    #NOTE: the blankline at the end is generated by subprocess.check_output, not by the script
    >>> print output[:500] #doctest: +NORMALIZE_WHITESPACE 
    genotype name window whole_gene_nsnps network_gene network_continent network_distance_definition n_datapoints cont_str degree closeness betweenness
    00100000000001000000000001000000001001000000000000  ALG10B  window_0    67  ALG10B  glob    1   572 afr_no_AMR_ASN_AFR_EUR  6   0.0213  15.0
    10100000010000000101010000010000001000001101010001  ALG10B  window_0    67  ALG10B  glob    1   390 afr_no_AMR_ASN_AFR_EUR  2   0.0208  4.0
    00100000010000000101000000000001001000000001010001  ALG10B  window_0    67  ALG10B  glob    1   247 afr_no_AMR_ASN_

    """
def testDebugReportHDF5():
    """
    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/ALG10B.genes -p test/test_data/params/debug.yaml --no-debug --data_basedir=./test/test_data -w 50 -t values_by_node --hdf5"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)

    #NOTE: the blankline at the end is generated by subprocess.check_output, not by the script
    >>> print output[:500] #doctest: +NORMALIZE_WHITESPACE 
    genotype name window whole_gene_nsnps network_gene network_continent network_distance_definition n_datapoints cont_str degree closeness betweenness
    00100000000001000000000001000000001001000000000000  ALG10B  window_0    67  ALG10B  glob    1   572 afr_no_AMR_ASN_AFR_EUR  6   0.0213  15.0
    10100000010000000101010000010000001000001101010001  ALG10B  window_0    67  ALG10B  glob    1   390 afr_no_AMR_ASN_AFR_EUR  2   0.0208  4.0
    00100000010000000101000000000001001000000001010001  ALG10B  window_0    67  ALG10B  glob    1   247 afr_no_AMR_ASN_
    """

def testDebugReportDistance2():
    """
    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/ALG10B.genes -p test/test_data/params/debug.yaml --no-debug --distance 2 --data_basedir=./test/test_data --windows_size 50 -t values_by_node"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)
    >>> print output[:500] #doctest: +NORMALIZE_WHITESPACE
    genotype name window whole_gene_nsnps network_gene network_continent network_distance_definition n_datapoints cont_str degree closeness betweenness
    00100000000001000000000001000000001001000000000000  ALG10B  window_0    67  ALG10B  glob    2   572 afr_no_AMR_ASN_AFR_EUR  6   0.0222  0.0
    10100000010000000101010000010000001000001101010001  ALG10B  window_0    67  ALG10B  glob    2   390 afr_no_AMR_ASN_AFR_EUR  7   0.0227  24.5
    00100000010000000101000000000001001000000001010001  ALG10B  window_0    67  ALG10B  glob    2   247 afr_no_AMR_ASN_
    """

def testDebugReportSmallWindow():
    """
    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/ALG10B.genes -p test/test_data/params/debug.yaml --no-debug --distance 2 --data_basedir=./test/test_data --windows_size 5 -t values_by_node"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)
    >>> print output[:500] #doctest: +NORMALIZE_WHITESPACE
    genotype name window whole_gene_nsnps network_gene network_continent network_distance_definition n_datapoints cont_str degree closeness betweenness
    00110   ALG10B  window_0    67  ALG10B  glob    2   597 afr_no_AMR_ASN_AFR_EUR  4   0.7 0.3333
    10010   ALG10B  window_0    67  ALG10B  glob    2   1079    afr_no_AMR_ASN_AFR_EUR  7   1.0 4.6667
    00000   ALG10B  window_0    67  ALG10B  glob    2   84  AMR_afr_no_AFR_EUR  3   0.6364  0.0
    11010   ALG10B  window_0    67  ALG10B  glob    2   338 afr_no_AMR_ASN_AFR_EUR  5   0.7778  0.6667
    11011   ALG10B  window_0    67  ALG10B  glob    
    """
def testFullReport():
    """
    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/ALG10B.genes -p params/default.yaml --no-debug --data_basedir=./test/test_data --windows_size 50 -t values_by_node"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)

    >>> print output[:500] #doctest: +NORMALIZE_WHITESPACE
    genotype name window whole_gene_nsnps network_gene network_continent network_distance_definition cont_str n_datapoints degree closeness betweenness
    00100000000001000000000001000000001001000000000000  ALG10B  window_0    67  ALG10B  glob    1   afr_no_AMR_ASN_AFR_EUR  572 6   0.0213  15.0
    10100000010000000101010000010000001000001101010001  ALG10B  window_0    67  ALG10B  glob    1   afr_no_AMR_ASN_AFR_EUR  390 2   0.0208  4.0
    00100000010000000101000000000001001000000001010001  ALG10B  window_0    67  ALG10B  glob    1   afr_no_AMR_ASN_AFR_

"""

def testDebugReportOverlapping():
    """
    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/ALG10B.genes -p test/test_data/params/debug.yaml --no-debug --data_basedir=./test/test_data -w 50 -t values_by_node -l"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)

    #NOTE: the blankline at the end is generated by subprocess.check_output, not by the script
    >>> print output[:500] #doctest: +NORMALIZE_WHITESPACE 
    genotype name window whole_gene_nsnps network_gene network_continent network_distance_definition n_datapoints cont_str degree closeness betweenness
    00011000001000000000010000000000010000000010010000  ALG10B  window_0    67  ALG10B  glob    1   572 afr_no_AMR_ASN_AFR_EUR  6   0.0227  15.0
    01001010101000000100000001010100000100000010000011  ALG10B  window_0    67  ALG10B  glob    1   390 afr_no_AMR_ASN_AFR_EUR  2   0.0217  3.0
    01001010001000000100000001010000000000010010000000  ALG10B  window_0    67  ALG10B  glob    1   247 afr_no_AMR_ASN_
"""





#!/usr/bin/env python
"""
"""
import random
import igraph
from numpy import mean, nan_to_num, nan
from pprint import pprint
import nose
import logging
logging.basicConfig(level=logging.DEBUG)


def get_averagePathLength(mygraph, weight_attribute=False, weighted=False):
    """Calculate the average path lenght of a graph
    
    split_by_component: split calculation by component to speed up computation (not tested) 
    
    See also my gist: https://gist.github.com/4124710
    """
#    return mean( [nan_to_num(m) for m in [mean([len(current_node_paths)-1 for current_node_paths in mygraph.get_shortest_paths(current_node) if not len(current_node_paths) <2]) for current_node in mygraph.vs()]   if m >= 0])
    # NOTE: for unconnected components, I need to find a way to calculate the average of the geodesic lengths in the components, as described in igraph.Graph.average_path_length.__elp__
#      calculated
    all_path_lengths = []
    num_path_lengths = 0
    current_lengths = []
    
    if weighted is True and weight_attribute not in mygraph.vs.attributes():
        print("attribute", weight_attribute, "does not exists")
   
    if weighted is True:
        current_lenghts = [path_len for path_len in sum(mygraph.shortest_paths(weights=mygraph.vs()[weight_attribute]), []) if path_len > 0 and path_len != float('inf')]
    else:
#        current_lenghts = [path_len for path_len in sum(mygraph.shortest_paths(), []) if path_len > 0 and path_len != float('inf')]
#        print mygraph.shortest_paths()
#        print (sum(mygraph.shortest_paths(), []))
#        current_lenghts = [path_len for path_len in sum(mygraph.shortest_paths(), []) if path_len > 0 and path_len != float('inf')]
        current_lengths = [path_len for path_len in sum(mygraph.shortest_paths(), []) if path_len > 0 and path_len != float('inf')]
#        print current_lengths

    logging.debug( current_lengths)
    av_path_length = mean(current_lengths)
    return av_path_length

known_cases = {
        'triple_path': {
            'comment': 'graph with a single path, two edges',
            'n_vertices': 3, 'edges': [(0, 1), (1, 2)],
#                              0    1    2
            'expected': mean((1,2, 1,1, 1,2)),
            },
        'quadruple_path': {
            'comment': 'graph with a single path, three edges',
            'n_vertices': 4, 'edges': [(0, 1), (1, 2), (2,3)],
#                               0       1       2       3
            'expected': mean((1,2,3,  1,1,2,  2,1,1,  3,2,1))
            },
        'unconnected_graph': {
            'comment': 'this graph contains unconnected nodes, av.path lenght should be 0 or nan',
            'n_vertices': 5, 'edges': [],
            'expected': nan
            },
        'pairs': {
            'comment': 'network where there are only pairs of edges. Av Path lenght should be 1',
            'n_vertices': 6, 'edges': [(0,1), (2,3), (4,5)],
            'expected': (1 + 1 + 1) / 3.
            },
        'onepair_onecomponent': {
            'comment': 'network with one unconnected node and two connected nodes',
            'n_vertices': 3, 'edges': [(1,2)],
            'expected': 1 
            },
        'onepair_someunconnected': {
            'comment': 'network two connected nodes and some unconnected components, to see the behaviour of the function when there are many unconnected components',
            'n_vertices': 10, 'edges': [(4,6)],
            'expected': 1
            },
        'two_components': {
            'comment': 'two components, each with 2 edges',
            'n_vertices': 10, 'edges': [(0,1), (1,2), (5,6), (6,7)],
#                              0      1      2
            'expected': mean((1,2,   1,1,   1,2)) #* 2 # NOTE: This has the same av.path lenght as triple_path, not the double
            },
        'two_unsymmetric_components1': {
            'comment': 'two components, one with one edge, and one with two', #TODO: finish test case
            'n_vertices': 5, 'edges': [(0,1), (2,3), (3,4)],
#                             0    1     2      3      4 
            'expected': mean((1,   1,   1,2,   1,1,   2,1))
            },
        'two_unsymmetric_components2': {
            'comment': 'two components, one with two edges, and one with three', #TODO: finish test case
            'n_vertices': 10, 'edges': [(0,1), (1,2), (5,6), (6,7), (7,8)],
#                              0      1      2       5        6        7        8
            'expected': mean((1,2,   1,1,   1,2,   1,2,3,   1,1,2,   2,1,1,   3,2,1))
            },
        'problematic1': {
            'comment': 'a graph generated randomly using the Erdos Renyi function, which gives problem because there are too many unconnected nodes',
            'n_vertices': 13,
            'edges':[(1, 4), (3, 7), (6, 10), (5, 11), (8, 11), (6, 12), (11, 12)],
#                             1   3    4         5             6         7         8              10            11             12
            'expected': mean((1,  1,   1,   1,2,2,3,4,     1,1,2,3,3,    1,    1,2,2,3,4,     1,2,3,4,4,    1,1,1,2,3,     1,1,2,2,2))
            },
        'star': {
            'comment': 'a star-like graph',
            'n_vertices': 6,
            'edges':[(1, 3), (2, 3), (3, 4), (3, 5)],
            'expected': mean((1,2,2,2,     2,1,2,2,     1,1,1,1,     2,2,1,2,     2,2,2,1))
            },
        'star_plus_asymmetric': {
            'comment': 'a star-like graph, plus a one-edge component',
            'n_vertices': 10,
            'edges':[(1, 3), (2, 3), (3, 4), (3, 5), (8, 9)],
            'expected': mean((1,2,2,2,     2,1,2,2,     1,1,1,1,     2,2,1,2,      2,2,2,1, 1, 1))
            }
        }
def test_AveragePathLenght_KnownGraph_vs_myimplementation():
    """
    Check if my custom implementation of Average Path Lenght gives the same results as what I calculated manually.
    """
    for (graphname, properties) in known_cases.items():
        mygraph = igraph.Graph()
        mygraph['name'] = graphname
        mygraph['comment'] = properties['comment']
        mygraph.add_vertices(properties['n_vertices'])
        mygraph.add_edges(properties['edges'])
        yield checkPathLenght_Correct_KnownGraph_vs_myimplementation, mygraph, properties['expected']

def checkPathLenght_Correct_KnownGraph_vs_myimplementation(mygraph, expected):
    print(mygraph)
    print(mygraph['name'])
    print(mygraph['comment'])
    print([e.tuple for e in mygraph.es()])
    for p in [mygraph.get_all_shortest_paths(n) for n in mygraph.vs()]: print p
    observed = nan_to_num(get_averagePathLength(mygraph))
    expected = nan_to_num(expected)
    expected_computationally = nan_to_num(mygraph.average_path_length())
    print "my_implementation:", observed, "\nmanual calculation:", expected, "\nigraph.Graph.average_path_lenght:", expected_computationally
    nose.tools.assert_almost_equals(observed, expected)

def test_AveragePathLenght_KnownGraph_vs_igraph():
    """
    Check if my custom implementation of Average Path Lenght gives the same results as the igraph.Graph.average_path_length function.
    """
    for (graphname, properties) in known_cases.items():
        mygraph = igraph.Graph()
        mygraph['name'] = graphname
        mygraph['comment'] = properties['comment']
        mygraph.add_vertices(properties['n_vertices'])
        mygraph.add_edges(properties['edges'])
        yield checkPathLenght_Correct_KnownGraph_vs_igraph, mygraph, properties['expected']

def checkPathLenght_Correct_KnownGraph_vs_igraph(mygraph, expected):
    """
    Check if my custom implementation of Average Path Lenght gives the same results as what I calculated manually.
    """
    print(mygraph)
    print(mygraph['name'])
    print(mygraph['comment'])
    print([e.tuple for e in mygraph.es()])
    for p in [mygraph.get_all_shortest_paths(n) for n in mygraph.vs()]: print p
    observed = nan_to_num(get_averagePathLength(mygraph))
    expected = nan_to_num(expected)
#    mygraph
    expected_computationally = nan_to_num(mygraph.average_path_length())
    print "my_implementation:", observed, "\nmanual calculation:", expected, "\nigraph.Graph.average_path_lenght:", expected_computationally
    for component in mygraph.components():
        subgraph = mygraph.subgraph(component)
        print "subgraph size", subgraph.vcount(), "number edges", subgraph.ecount(), "subgraph average path length:", subgraph.average_path_length()
        print ([[len(node_paths)-1 for node_paths in subgraph.get_shortest_paths(node)] for node in subgraph.vs()])
#    assert False
    nose.tools.assert_almost_equals(observed, expected_computationally)

def test_AveragePathLenght_RandomGraph():
    """Generate some random graphs, and for each of them, check if my implementation of average path lenght gives the same results as the igraph.Graph.average_path_lenght function
    """

    random.seed(10)

    for randomgraph_id in range(10):
        n = random.randrange(2, 20)
#        m = random.randrange(2, (n-1)**2)
        p = random.random()
        yield check_PathLenght_Correct_RandomGraph, n, p

def check_PathLenght_Correct_RandomGraph(nodes, p):
    randomgraph = igraph.Graph.Erdos_Renyi(nodes, p)
    mygraph = randomgraph
    print(randomgraph)
    print randomgraph.get_adjedgelist()
    print [edge.tuple for edge in randomgraph.es]

#    print( "paths", ([[("node ID:", current_node.index, "node paths:", current_node_paths) for current_node_paths in mygraph.get_shortest_paths(current_node) if not len(current_node_paths) <2] for current_node in mygraph.vs()]))
    print ([(current_node.index, "paths:", [current_node_paths for current_node_paths in mygraph.get_shortest_paths(current_node) if not len(current_node_paths) < 2]) for current_node in mygraph.vs()])
    print
    print "lenght of paths -1", ([(current_node.index, "paths:", [len(current_node_paths)-1 for current_node_paths in mygraph.get_shortest_paths(current_node) if not len(current_node_paths) < 2]) for current_node in mygraph.vs()])
    print
    print "mean lenghts -1", ([(current_node.index, "paths:", mean([len(current_node_paths)-1 for current_node_paths in mygraph.get_shortest_paths(current_node) if not len(current_node_paths) < 2])) for current_node in mygraph.vs()])
    print "mean lenghts -1", ([mean([len(current_node_paths)-1 for current_node_paths in mygraph.get_shortest_paths(current_node) if not len(current_node_paths) < 2]) for current_node in mygraph.vs()])
    print
#    print , ([m for m in [[len(current_node_paths)-1 for current_node_paths in mygraph.get_shortest_paths(current_node) if not len(current_node_paths) <2] for current_node in mygraph.vs()]   if m >= 0] )
#    print
    print "mean lenghts -1", (mean( [m for m in [mean([len(current_node_paths)-1 for current_node_paths in mygraph.get_shortest_paths(current_node) if not len(current_node_paths) <2]) for current_node in mygraph.vs()]   if m >= 0] ))
    print
#    pprint (mean( [m for m in [mean([len(p) for p in randomgraph.get_shortest_paths(n) if not len(p) <2]) for n in randomgraph.vs()]   if m >= 0] ))
#    pprint (mean( [m for m in [mean([len(p) for p in randomgraph.get_shortest_paths(n) if not len(p) <2]) for n in randomgraph.vs()]   if m >= 0] ))
#    pprint (mean( [m for m in [mean([len(p) for p in randomgraph.get_shortest_paths(n) if not len(p) <2]) for n in randomgraph.vs()]   if m >= 0] ))

    observed = nan_to_num(get_averagePathLength(randomgraph))
    expected = nan_to_num(randomgraph.average_path_length())
    print observed, expected
#    assert False
    nose.tools.assert_almost_equals(observed, expected)












if __name__ == '__main__':
    pass




#!/usr/bin/env python
"""
"""
from test_SubgraphByPop import SimpleNetwork
import binary_to_network
import nose
import numpy

class SimpleNetworkFromFile(SimpleNetwork):
    binary_genotypes_file = 'test/test_data/binary_genotypes/MOGS.binary'
    individuals_file = 'test/test_data/phase1_integrated_calls.20101123.ALL.panel.individuals'

    expected_str_by_pop = {
                    'glob' : "Genotype Network (name = 'MOGS', chromosome lenght = 7, |V| = 13, |E| = 14)",
                    "AMR": "Genotype Network (name = 'MOGS_AMR', chromosome lenght = 7, |V| = 10, |E| = 8)",
                    "ASN": "Genotype Network (name = 'MOGS_ASN', chromosome lenght = 7, |V| = 7, |E| = 6)",
                    "EUR": "Genotype Network (name = 'MOGS_EUR', chromosome lenght = 7, |V| = 8, |E| = 5)",
                    "AFR": "Genotype Network (name = 'MOGS_AFR', chromosome lenght = 7, |V| = 8, |E| = 7)"
    }
    expected_report_by_pop = {
                    'glob': "MOGS glob 7 2184 13 14 1 5 0.1795 168.0 2.7692 2.1538 5 10.6154 0.3729",
                    "AMR": "MOGS_AMR AMR 7 362 10 8 3 3 0.1778 36.2 1.5625 1.6 3 0.9 0.1456",
                    "ASN": "MOGS_ASN ASN 7 572 7 6 2 2 0.2857 81.7143 1.3333 1.7143 2 0.4286 0.221",
                    "EUR": "MOGS_EUR EUR 7 758 8 5 3 2 0.1786 94.75 1.4444 1.25 3 0.5 0.1731",
                    "AFR": "MOGS_AFR AFR 7 370 8 7 1 4 0.25 46.25 2.25 1.75 4 4.375 0.465"
    }
    expected_nodes_by_pop = {
                    'glob': ['0000000', '0000001', '0000100', '0011000', '0000010', '0011001', '0011100', '0111000', '0011010', '1010000', '0000110', '0000101', '0010000'],
                    'AMR': ['0000000', '0000001', '0000100', '0011000', '0000010', '0011001', '0011100', '0111000', '1010000', '0000110'],
                    'ASN': ['0000000', '0000001', '0000100', '0011000', '0011001', '0011100', '0000101'],
                    'EUR': ['0000000', '0000001', '0000100', '0011000', '0000010', '0011001', '0011100', '1010000'],
                    'AFR': ['0000000', '0000100', '0011000', '0011100', '0111000', '0011010', '1010000', '0010000']
    }


    expected_n_datapoints_by_pop = {
                    'glob': [956, 55, 26, 934, 44, 7, 42, 63, 18, 34, 2, 1, 2],
                    'AMR': [1, 2, 3, 4, 6, 9, 10, 41, 83, 203],
                    'ASN': [1, 2, 2, 2, 18, 98, 449],
                    'EUR': [1, 3, 4, 5, 13, 41, 105, 586],
                    'AFR': [1, 2, 12, 16, 27, 38, 53, 221]
#                    'AFR': [69, 2, 2, 297, 13, 59, 18, 30, 2]
    }

    expected_diameter_by_pop = {
                    "glob" : 5,
                    "AMR": 3,
                    "ASN": 2,
                    "EUR": 2,
                    "AFR": 4,
    }
    expected_avdegree_by_pop = {
                    'glob': 2.1538,
                    'AMR': 1.6,
                    'ASN': 1.7143,
                    'EUR': 1.25,
                    'AFR': 1.75
    }
    expected_av_w_degree_by_pop = {
                    'glob': 0.3151,
                    'AMR': 0.2715,
                    'ASN': 0.2807,
                    'EUR': 0.3354,
                    'AFR': 0.3625
    }


    individuals_dict = binary_to_network.parse_individuals_file(individuals_file, pickle_from_file_if_possible=False) #TODO: de-activate pickling of this variable
    all_genotypes, individuals, individuals_dict, gene_name = binary_to_network.parse_binary_file(
        binary_genotypes_file, individuals_dict)
    network = binary_to_network.generate_network(all_genotypes, individuals, individuals_dict, gene_name) 
#    network = binary_to_network.generate_network(binary_genotypes_file, individuals_dict) 

    def setUp(self):
#        delattr(self, test_NodesIndividuals)
#        print self.network.vs['genotype']
        self.subgraphs = {}
        self.network.__calculate_network_properties__()
        self.pops = ("AMR", "ASN", "EUR", "AFR")
        for pop in self.pops:
            self.subgraphs[pop] = self.network.subgraph_by_continent(pop)
            self.subgraphs[pop].__calculate_network_properties__()
            print pop, self.subgraphs[pop].vs()['genotype']

    def test_NodesIndividuals(self):
        self.skipTest("Individuals test skipped, because there are too many individuals")

class DPAGT1SimpleNetworkFromHDF5File(SimpleNetwork):
    hdf5_genotypes_file = 'test/test_data/hdf5/DPAGT1_genotypes.hdf5'
    individuals_file = 'test/test_data/phase1_integrated_calls.20101123.ALL.panel.individuals'

    expected_str_by_pop = {
                    'glob' : "Genotype Network (name = 'DPAGT1', chromosome lenght = 11, |V| = 20, |E| = 15)",
                    "AMR": "Genotype Network (name = 'DPAGT1_AMR', chromosome lenght = 11, |V| = 7, |E| = 3)",
                    "ASN": "Genotype Network (name = 'DPAGT1_ASN', chromosome lenght = 11, |V| = 9, |E| = 2)",
                    "EUR": "Genotype Network (name = 'DPAGT1_EUR', chromosome lenght = 11, |V| = 7, |E| = 3)",
                    "AFR": "Genotype Network (name = 'DPAGT1_AFR', chromosome lenght = 11, |V| = 14, |E| = 11)"
    }
    expected_report_by_pop = {
                    'glob': "DPAGT1 glob 11 2184 20 15 6 7 0.0789 109.2 3.25 1.5 5 10.35 0.0973",
                    'AMR': "DPAGT1_AMR AMR 11 362 7 3 4 2 0.1429 51.7143 1.25 0.8571 2 0.1429 0.1723",
                    "ASN": "DPAGT1_ASN ASN 11 572 9 2 7 1 0.0556 63.5556 1.0 0.4444 1 0.0 0.1173",
                    "EUR": "DPAGT1_EUR EUR 11 758 7 3 4 2 0.1429 108.2857 1.25 0.8571 2 0.1429 0.1723",
                    "AFR": "DPAGT1_AFR AFR 11 370 14 11 4 6 0.1209 26.4286 2.8364 1.5714 5 7.2143 0.1615"
    }
    expected_nodes_by_pop = {
                    'glob': ['00000000000', '00000000100', '00000000101', 
                        '00000000111', '00000001111', '00000110101', 
                        '00001000100', '00010000100', '00010000101', 
                        '00110000101', '00110100101', '00110110101', 
                        '00111001111', '01110000101', '01110001000', 
                        '01110001110', '01110001111', '01110110101', 
                        '10000000100', '10000100101'],
                    'AMR': ['00000000100', '00110110101', '01110001111', 
                        '00000000000', '00110000101', '00110100101', 
                        '00000000111'],
                    'ASN': ['00000000100', '00110110101', '01110001111', 
                        '00000000000', '00110000101', '00000110101', 
                        '01110001000', '01110110101', '00000001111'], 
                    'EUR': ['00000000100', '00110110101', '01110001111', 
                        '00000000000', '10000100101', '00000000101', 
                        '01110001110'], 
                    'AFR': ['00000000100', '00110110101', '01110001111', 
                        '00000000000', '10000100101', '00110000101', 
                        '00111001111', '00010000100', '00110100101', 
                        '00000000101', '10000000100', '01110000101', 
                        '00001000100', '00010000101']
    }


    expected_n_datapoints_by_pop = {
                    'glob': [952, 164, 721, 248, 25, 11, 1, 20, 1, 3, 7, 8, 14, 1, 2, 1, 1, 2, 1, 1],
                    'AMR': [1, 1, 2, 19, 24, 100, 215],
                    'ASN': [67, 64, 279, 146, 6, 1, 1, 7, 1],
                    'EUR': [367, 45, 288, 50, 6, 1, 1],
                    'AFR': [225, 22, 38, 27, 15, 3, 17, 3, 7, 8, 1, 1, 2, 1]
    }

    expected_diameter_by_pop = {
                    "glob" : 7,
                    "AMR": 2,
                    "ASN": 1,
                    "EUR": 2,
                    "AFR": 6,
    }
    expected_avdegree_by_pop = {
                    'glob': 1.5,
                    'AMR': 0.8571,
                    'ASN': 0.4444,
                    'EUR': 0.8571,
                    'AFR': 1.5714
    }
    expected_av_w_degree_by_pop = {
                    'glob': 0.1413,
                    'AMR': 0.1034,
                    'ASN': 0.0552,
                    'EUR': 0.2024,
                    'AFR': 0.2367
    }

    individuals_dict = binary_to_network.parse_individuals_file(
        individuals_file, pickle_from_file_if_possible=False)
    hdf5_genotypes_file = open(hdf5_genotypes_file, 'r')
    all_genotypes, individuals, individuals_dict, gene_name = binary_to_network.parse_hdf5_file(
        hdf5_genotypes_file, individuals_dict)
    network = binary_to_network.generate_network(all_genotypes, individuals, individuals_dict, gene_name, use_hdf5=True) 
#    network = binary_to_network.generate_network(binary_genotypes_file, individuals_dict) 

    def setUp(self):
#        delattr(self, test_NodesIndividuals)
#        print self.network.vs['genotype']
        self.subgraphs = {}
        self.network.__calculate_network_properties__()
        self.pops = ("AMR", "ASN", "EUR", "AFR")
        for pop in self.pops:
            self.subgraphs[pop] = self.network.subgraph_by_continent(pop)
            self.subgraphs[pop].__calculate_network_properties__()
            print pop, self.subgraphs[pop].vs()['genotype']

    def test_NodesIndividuals(self):
        self.skipTest("Individuals test skipped, because there are too many individuals")


class SimpleNetworkFromHDF5File(SimpleNetwork):
    hdf5_genotypes_file = 'test/test_data/hdf5/MOGS_genotypes.hdf5'
    individuals_file = 'test/test_data/phase1_integrated_calls.20101123.ALL.panel.individuals'

    expected_str_by_pop = {
                    'glob' : "Genotype Network (name = 'MOGS', chromosome lenght = 7, |V| = 13, |E| = 14)",
                    "AMR": "Genotype Network (name = 'MOGS_AMR', chromosome lenght = 7, |V| = 10, |E| = 8)",
                    "ASN": "Genotype Network (name = 'MOGS_ASN', chromosome lenght = 7, |V| = 7, |E| = 6)",
                    "EUR": "Genotype Network (name = 'MOGS_EUR', chromosome lenght = 7, |V| = 8, |E| = 5)",
                    "AFR": "Genotype Network (name = 'MOGS_AFR', chromosome lenght = 7, |V| = 8, |E| = 7)"
    }
    expected_report_by_pop = {
                    'glob': "MOGS glob 7 2184 13 14 1 5 0.1795 168.0 2.7692 2.1538 5 10.6154 0.3729",
                    "AMR": "MOGS_AMR AMR 7 362 10 8 3 3 0.1778 36.2 1.5625 1.6 3 0.9 0.1456",
                    "ASN": "MOGS_ASN ASN 7 572 7 6 2 2 0.2857 81.7143 1.3333 1.7143 2 0.4286 0.221",
                    "EUR": "MOGS_EUR EUR 7 758 8 5 3 2 0.1786 94.75 1.4444 1.25 3 0.5 0.1731",
                    "AFR": "MOGS_AFR AFR 7 370 8 7 1 4 0.25 46.25 2.25 1.75 4 4.375 0.465"
    }
    expected_nodes_by_pop = {
                    'glob': ['0000000', '0000001', '0000100', '0011000', '0000010', '0011001', '0011100', '0111000', '0011010', '1010000', '0000110', '0000101', '0010000'],
                    'AMR': ['0000000', '0000001', '0000100', '0011000', '0000010', '0011001', '0011100', '0111000', '1010000', '0000110'],
                    'ASN': ['0000000', '0000001', '0000100', '0011000', '0011001', '0011100', '0000101'],
                    'EUR': ['0000000', '0000001', '0000100', '0011000', '0000010', '0011001', '0011100', '1010000'],
                    'AFR': ['0000000', '0000100', '0011000', '0011100', '0111000', '0011010', '1010000', '0010000']
    }


    expected_n_datapoints_by_pop = {
                    'glob': [956, 55, 26, 934, 44, 7, 42, 63, 18, 34, 2, 1, 2],
                    'AMR': [1, 2, 3, 4, 6, 9, 10, 41, 83, 203],
                    'ASN': [1, 2, 2, 2, 18, 98, 449],
                    'EUR': [1, 3, 4, 5, 13, 41, 105, 586],
                    'AFR': [1, 2, 12, 16, 27, 38, 53, 221]
#                    'AFR': [69, 2, 2, 297, 13, 59, 18, 30, 2]
    }

    expected_diameter_by_pop = {
                    "glob" : 5,
                    "AMR": 3,
                    "ASN": 2,
                    "EUR": 2,
                    "AFR": 4,
    }
    expected_avdegree_by_pop = {
                    'glob': 2.1538,
                    'AMR': 1.6,
                    'ASN': 1.7143,
                    'EUR': 1.25,
                    'AFR': 1.75
    }
    expected_av_w_degree_by_pop = {
                    'glob': 0.3151,
                    'AMR': 0.2715,
                    'ASN': 0.2807,
                    'EUR': 0.3354,
                    'AFR': 0.3625
    }

    individuals_dict = binary_to_network.parse_individuals_file(
        individuals_file, pickle_from_file_if_possible=False)
    hdf5_genotypes_file = open(hdf5_genotypes_file, 'r')
    all_genotypes, individuals, individuals_dict, gene_name = binary_to_network.parse_hdf5_file(
        hdf5_genotypes_file, individuals_dict)
    network = binary_to_network.generate_network(all_genotypes, individuals, individuals_dict, gene_name, use_hdf5=True) 
#    network = binary_to_network.generate_network(binary_genotypes_file, individuals_dict) 

    def setUp(self):
#        delattr(self, test_NodesIndividuals)
#        print self.network.vs['genotype']
        self.subgraphs = {}
        self.network.__calculate_network_properties__()
        self.pops = ("AMR", "ASN", "EUR", "AFR")
        for pop in self.pops:
            self.subgraphs[pop] = self.network.subgraph_by_continent(pop)
            self.subgraphs[pop].__calculate_network_properties__()
            print pop, self.subgraphs[pop].vs()['genotype']

    def test_NodesIndividuals(self):
        self.skipTest("Individuals test skipped, because there are too many individuals")


#from nose.plugins.attrib import attr
#@attr('slow')
@nose.plugins.attrib.attr('slow')
def test_SubgraphingWorks():
    """check that the function subgraph_by_continent works, without checking all the details as in the other TestCase of this file.
    
    NOTE: RPN2 is very slow"""
    individuals_file = 'test/test_data/phase1_integrated_calls.20101123.ALL.panel.individuals'
    individuals_dict = binary_to_network.parse_individuals_file(individuals_file, pickle_from_file_if_possible=False) #TODO: de-activate pickling of this variable

    genes = 'FUT8 TUSC3 RPN2'.split()
    pops = ("AMR", "ASN", "EUR", "AFR")
    for gene in genes: 
        for pop in pops:
            binary_genotypes_file = 'test/test_data/binary_genotypes/{0}.binary'.format(gene)
            all_genotypes, individuals, individuals_dict, gene_name = binary_to_network.parse_binary_file(
                binary_genotypes_file, individuals_dict)
            network = binary_to_network.generate_network(all_genotypes, individuals, individuals_dict, gene_name) 
#            network = binary_to_network.generate_network(binary_genotypes_file, individuals_dict) 
            yield check_Subgraphing, pop, network, individuals_dict

def check_Subgraphing(pop, network, individuals_dict):
    subnetwork = network.subgraph_by_continent(pop)
    assert (subnetwork.vcount() > 0)





if __name__ == '__main__':
    import logging
    import doctest
    logging.basicConfig(level=logging.DEBUG)
    doctest.testmod()





#!/usr/bin/env python
"""
Get chromosome coordinates from UCSC tables

Usage:

    python get_gene_coords.py -l genes.txt -d hg19

genes.txt is a file containing a list of Gene Ids to be retrieved, one per line.

Reference: http://biostar.stackexchange.com/questions/3121/genomic-cordinates-from-ucsc

Script taken from another repository: https://bitbucket.org/dalloliogm/ucsc-fetch
"""

import subprocess
#import sys
import logging
import optparse

def get_options():
    parser = optparse.OptionParser(usage="python {} -l genes.txt -d hg19".format(__file__), description=__doc__)
    parser.add_option('-l', '-g', '-f', '--list', '--list_of_genes', '--genes', dest='genes', 
            help='file containing the list of gene. One symbol per line', default=False)
    parser.add_option('-d', '--database', dest='database',
            help='database version (e.g. hg18 or hg19). Default: hg19', default='hg19')
    parser.add_option('-u', '--debug', dest='debug', action='store_true', default=True)
    (options, args) = parser.parse_args()
    
    if options.debug is True:
        logging.basicConfig(level=logging.DEBUG, filename='logs/get_gene_coords.log')

    if (options.genes is False) and (len(args) == 0):
        parser.print_help()
        parser.error('get_gene_coords.py: genes file not defined.')
#    print args
#    print options.genes
    try:
        genes_path = ''
        if options.genes is not False:
            genes_path = options.genes
        elif args != '':
            genes_path = args[0]
#        print genes_path
#        if genes_path != '':
#            genes_h = open(genes_path, 'r')
    except:
        print __doc__
        parser.error("Can not open genes file")

    (genes, gene_order) = read_genes_file(genes_path)
    logging.debug(genes)

    return (genes, gene_order, options.database)

def read_genes_file(genes_path):
    """
    Read a genes file and returns a list of lists
    
    Example genes file
    ----------------------

    ..
        
        #gene    pathway subpathway
        DPAGT1  glycosylation   precursor
        ALG2  glycosylation   precursor
        ALG1  glycosylation   precursor


    Example output list
    -------------------

    ..
        [ 
            ("DPAGT1", "glycosylation", "precursor"),
            ("ALG2", "glycosylation", "precursor"),
            ("ALG1", "glycosylation", "precursor"),
        ]
    """
    genes = {}
    gene_order = []
    with open(genes_path) as genes_h:
        for line in genes_h:
#            print 'line "' + line.strip() + '"' 
            if not line.strip() == '' and not line.startswith('#'):
                fields = line.split()
                gene_name = fields[0]
                if genes.has_key(gene_name):
                    raise ValueError("Duplicated line in gene list file {0}".format(genes_path))
                genes[gene_name] = tuple(fields)
                gene_order.append(gene_name)
#    print genes
    return genes, gene_order

def query_ucsc(genes, gene_order, database):
    """
    Query UCSC MySQL database to get coordinates of a list of genes.

    Print results to STDOUT
    """
    # QUERYING UCSC
    genes_str = '("' + '", "'.join([gene for gene in genes.keys()]) + '")'
    command = """mysql  -h  genome-mysql.cse.ucsc.edu -A -u genome -D %s -e 'select distinct X.geneSymbol,G.chrom,G.txStart,G.txEnd from knownGene as G join kgXref as X on X.kgId=G.name where X.geneSymbol in %s ' """ % (database, genes_str)
#    command = """mysql  -h  genome-mysql.cse.ucsc.edu -A -u genome -D %s -e 'select distinct K.geneSymbol,G.chrom,G.txStart,G.txEnd from refGene as G join kgXref as X on X.kgId=G.name2 where X.geneSymbol in %s ' """ % (database, genes_str)
    logging.debug(command)

    # PARSING RESULTS to remove redundant entries (TODO: use Mysql function)

    # I don't have a recent version of the subprocess installed, so instead of subprocess.check_output I have to use a temporary file.
    temp_output = subprocess.check_output(command, shell=True)

    all_genes = {}
    for line in temp_output.split('\n')[1:]:
#        print "line: '" + line + "'", line.strip() == ''
        if not line.strip() == '':
#            print "line", line
            (gene, chrom, start, end) = line.split()
            (name, pathway, subpathway) = genes[gene]
#            print name, pathway, subpathway
            start = int(start)
            end = int(end)
#            logging.debug(start < end)
            if not all_genes.has_key(gene):
                all_genes[gene] = [gene, pathway, subpathway, chrom, start, end]
            else:
                if start < all_genes[gene][4]:
                    logging.debug((gene, "(start)", start, all_genes[gene][4]))
                    all_genes[gene][4] = start

                if end > all_genes[gene][5]:
                    logging.debug((gene, "(end)", end, all_genes[gene][5]))
                    all_genes[gene][5] = end

    # PRINTING OUTPUT

#    for (gene, values) in all_genes.items():
    for gene_name in gene_order:
        if all_genes.has_key(gene_name):
            values = all_genes[gene_name]
            print '%s\t%s\t%s\t%s\t%s\t%s' % tuple(values)
        else:
            logging.warning("Could not find coordinates for gene {}".format(gene_name))
#    print ("output saved to {}".format(outputfile))

def main():
    (genes, gene_order, database) = get_options()
#    print genes, database
    query_ucsc(genes, gene_order, database)

if __name__ == '__main__':
    main()




#!/usr/bin/env python
"""
This script will call binary_to_network.py on all the genes of a pathway, and 
create different types of reports for all the properties (see the option 
--report_type for a list of all possible report types).

It requires a single argument, a genes file containing all the genes to be 
included in the analysis. 
This is the same file used in get_gene_coords.py, and is also described 
in the main README.rst of this repository.

Morever, this script will assume that files containing the genotypes in 
binary format (produced by filteredvcf_to_binary.py) are stored in the 
data/binary directory. 
If one of these files is missing or empty, a warning message will be 
sent to logs/generate_network_report.txt

"""
import get_gene_coords
import binary_to_network
import logging
import optparse
import os
import random
import numpy
try:
    from GenotypeNetwork import GenotypeNetworkInitError
    import GenotypeNetwork
except ImportError:
    from src.GenotypeNetwork import GenotypeNetworkInitError as GenotypeNetworkInitError
    from src.GenotypeNetwork import GenotypeNetwork
import yaml
import re
#import gc
#import multiprocessing

def get_options():
    parser = optparse.OptionParser(usage="python %prog -f genes_file.genes", 
            description=__doc__)
    config_group = optparse.OptionGroup(parser, "Configuration options. Required")
    config_group.add_option('-f', '-g', '-v', '--genes', dest='genes_file', 
            help='File containing the list of genes to be analyzed. Required. Default: %default. ', default=False)
    config_group.add_option('-p', '--params', dest='params_file', 
            help='File containing the parameters of the analysis. In particular, the fields to be included in the report. Required. Default: %default', 
            default="params/default.yaml")
    config_group.add_option('--report_type', '-t', '-r', dest='report_type', 
            help='Type of report to produce.  \nPossible values: "interpop_comparison", "values_by_gene", "values_by_node", "individuals", and "individuals_line". Default: %default',
            choices=("values_by_gene", "values_by_node", "interpop_comparison", "individuals", "individuals_line"),
            default="values_by_gene")
    parser.add_option_group(config_group)
    network_definition_group = optparse.OptionGroup(parser, "Network Definition options")
    network_definition_group.add_option('-n', '-d', '--distance', dest='distance', type='int',
            help='Distance between nodes in the network. Two nodes will be considered being neighbors if they have at least _distance_ number of differences. Default: %default',
            default=1)
    network_definition_group.add_option('-w', '--windows_size', '--windows', dest='windows_size', type='int',
            help='Split the gene in non-overlapping windows of a given size. If not provided, windows approach won\'t be used. Default: %default', 
            default=0)
    network_definition_group.add_option('-i', '--individuals', '--number_individuals', dest='n_individuals', type='int',
            help='Number of individuals to use. If it is less than the actual number of samples, only a randomly chosen subset of these is used. This value is applied only to the population networks, not to the global one. Also, note that n_individuals = n_haplotypes/2. Default: %default',
            default=0)
    network_definition_group.add_option('-l', '--overlapping', '--overlapping_windows', dest='overlapping', action='store_true',
            help='Should sliding windows be overlapping? If True, each window will start from the second base of the previous window. Default: %default',
            default=False)
    network_definition_group.add_option('--random_seed', dest='random_seed', type='int',
            help='Random seed, for sampling individuals. If 0, a random number is used. Default: %default',
            default=0)
    network_definition_group.add_option('-s', '--number_subsampling', dest='n_subsampling', type='int',
            help='How many times to resample individuals, if the -i option is not 0. Default: %default', 
            default=1)
    parser.add_option_group(network_definition_group)
    performances_group = optparse.OptionGroup(parser, "Options to save/read files from graphml and pickle")
    performances_group.add_option('--write_pickle', dest='write_pickle', action='store_true',
            help='Write networks to pickle files, to the data/pickle folder', 
            default=False)
    performances_group.add_option('--write_graphml', dest='write_graphml', action='store_true',
            help='Write networks to graphml files, to the data/graphml folder', 
            default=False)
    performances_group.add_option('--overwrite_existing', dest='overwrite_existing', action='store_true',
            help='Used in combination with --write_graphml and --write_pickle, overwrite files if they already exist',
            default=False)
    performances_group.add_option('--read_graphml_if_possible', dest='read_graphml_if_possible', action='store_true',
            help='Attempt to read networks from graphml files, if they exists. Otherwise, generate new network', 
            default=False)
    performances_group.add_option('--data_basedir', dest='data_basedir', type='str',
            help='Path to the folder where graphmlfiles and pickle files will be stored or loaded from. Example: ./data/',
            default='./data/')
    performances_group.add_option('--hdf5', '--h5',
            help='Use HDF5 internal storage. More memory efficient, but requires the h5py library to be installed.',
            dest='use_hdf5', action='store_true', default=False)
    parser.add_option_group(performances_group)
    notimplemented_group = optparse.OptionGroup(parser, "Options not implemented yet")
    notimplemented_group.add_option('--parallelize', dest='parallelize', action='store_true', 
            help='use multiple processors for generating the report',
            default=False)
    parser.add_option_group(notimplemented_group)
    parser.add_option('-u', '--no-debug', dest='debug', action='store_false', 
            help="disable debugging messages, which are enabled by default", default=True)
    (options, args) = parser.parse_args()
    
    if options.parallelize is True:
        raise NotImplementedError("parallelization wasn't working correctly, so I removed the code for it.")
    if options.report_type in ("interpop_comparison", ):
        raise NotImplementedError("interpop comparison report not implemented yet")
    if options.debug is True:
        logging.basicConfig(level=logging.DEBUG)

    if (options.genes_file is False) and (len(args) == 0):
        parser.print_help()
        parser.error('generate_network_report.py error: no genes file provided')
    try:
        genes_file_path = ''
        if options.genes_file is not False:
            genes_file_path = options.genes_file
        elif args != '':
            network_from_graphml = args[0]
        if os.stat(genes_file_path).st_size == 0:
            parser.error("file {0} is empty".format(genes_file_path))
    except:
        print __doc__
        parser.error("Can not open genotypes file {0}".format(options.genes_file))
    options.genes_file_path = genes_file_path

    return (options)

def generate_report_windows(options, report_type='csv_report'):
    """
    Parse a file containing binary genotypes, generate a Genotype Network for each population, and write a report showing a resume of the network properties.

    +++++++++++++++++
    Parameters
    +++++++++++++++++

    - options: see the documentation for this script (--help)
    - columns included in the report file: these are defined in the params file.

    ++++++++++++++++++++++
    Example output report:
    ++++++++++++++++++++++

    There are three possible output report:

        * values_by_gene report
        * values_by_node report
        * interpop_comparison report

    ------------------------
    values_by_gene report 

    This report shows one line per every window/gene.
     
    ::

        gene continent distance_definition n_snps n_datapoints av_datapoints_per_node n_vertices n_edges n_components av_path_length diameter av_degree window whole_gene_nsnps
        ALG3 glob 1 30 2184 42.8235 51 50 6 3.7783 8 1.9608 8 0.0319 window_0 76
        ALG3 glob 1 30 2184 59.027 37 31 9 3.2544 7 1.6757 12 0.0517 window_1 76
        ALG3 AFR 1 30 492 13.6667 36 31 6 2.9438 6 1.7222 5 window_0 76
        ALG3 AFR 1 30 492 21.3913 23 15 8 3.1481 7 1.3043 5 window_1 76
        ALG3 ASN 1 30 572 63.5556 9 4 5 1.2 2 0.8889 2 window_0 76


    ------------------------
    values_by_node report 

    This report shows one line per every window/gene.
     
    ::
        
        genotype    degree  closeness
        01000       2       0.6
        00000       1       0.3
        01001       1       0.3 


    ------------------------
    interpop_comparison

    Not implemented yet
     
    """
    random_seed = options.random_seed
    original_random_seed = random_seed
    if random_seed is False:
        random_seed = random.random()
    genes = get_gene_coords.read_genes_file(options.genes_file_path)[1]
    individuals_dict = binary_to_network.parse_individuals_file()

    params_file = yaml.load(open(options.params_file, 'r'))
    pops = params_file['data']['populations']
    report_fields = []
    if report_type == 'values_by_gene':
        report_fields.extend(params_file['networkproperties_report_fields'])
        report_fields.append('window')
        report_fields.append('whole_gene_nsnps')
    elif report_type == 'values_by_node':
        report_fields.extend(['genotype', 'name', 'window', 'whole_gene_nsnps'])
        report_fields.extend(params_file["allvalues_report_fields"])

    if options.use_hdf5 is True:
        binaryfile_basepath = options.data_basedir + '/hdf5/{0}_genotypes.hdf5'
    else:
        binaryfile_basepath = options.data_basedir + '/binary_genotypes/{0}.binary'
#    graphmlfile_basepath = options.data_basedir + '/graphml/{}_{}_distance{}.graphml'
#    picklefile_basepath = options.data_basedir + '/pickles/{}_{}_distance{}.pickle'
#    networks = {}
    header = ' '.join(report_fields) 

    print header
#    report_output = header + '\n'
    windows_size = options.windows_size
    if options.overlapping is True:
        step = 1
    else:
        step = windows_size

    if options.n_individuals > 1:
        pops.insert(0, "global_sub")
    for gene in genes:
#        logging.debug("generated network report for gene {0}".format(gene))
        if gene not in ('gene',):
            skip_gene = False
            allpops_binaryfilepath = binaryfile_basepath.format(gene)
            try:
                allpops_binaryfile = open(allpops_binaryfilepath, 'r')
            except IOError:
                skip_gene = True
                logging.warn("The Binary File for {0} is not found.".format(allpops_binaryfilepath))
#                continue
            if skip_gene is False:
                for subsample_id in range(options.n_subsampling):
                    random_seed += 1  #TODO: this is the code that breaks the subsampling 
                    binary_to_network.print_report_and_subgraphs(allpops_binaryfile, gene, individuals_dict, 
                            pops, report_type, report_fields, options, random_seed=random_seed)
        random_seed = original_random_seed
#        logging.debug("gene {0} completed".format(gene))
#   return report_output


def read_graphml(input_graphmlfile, read_graphml_if_possible, binaryfile, individuals_dict, distance, gene):
    current_network = None
    if read_graphml_if_possible is True:    #TODO: convert back to GenotypeNetwork object
        logging.info("Trying to read " + input_graphmlfile)
        try:
            current_network = GenotypeNetwork.network_from_graphml(input_graphmlfile)
        except:
            logging.warn("could not read file from graphml, " + input_graphmlfile)
    if current_network is None:
        logging.info("Generating network from " + binaryfile)
        try:
            current_network = binary_to_network.generate_network(binaryfile, 
                          individuals_dict=individuals_dict, distance=distance)
        except:
            logging.error("Failed reading binary {0} file for {1}".format(binaryfile, gene))
            current_network = False
#    if current_network is None:
#        continue
    return current_network
def write_graphml(output_graphmlfile, current_network, overwrite_existing):
    """Write a network to a graphmlfile, if it doesn't exist already or overwrite_existing is True"""
    force_graphml_write = False
    if not os.path.exists(output_graphmlfile):
        force_graphml_write = True
    elif (overwrite_existing is True) and (os.stat(output_graphmlfile).st_size > 0):
        force_graphml_write = True
    if force_graphml_write is True:
        logging.info("wrote graphml to " + output_graphmlfile)
        try:
            current_network.write_graphml(output_graphmlfile)
        except TypeError:
            logging.warn("Error saving {0} file".format(output_graphmlfile))
    else:
        logging.info("graphml file {0} not written, because it already exists and the --overwrite option is not True".format(output_graphmlfile))

def write_pickle(output_picklefile, current_network, overwrite_existing):
    """Write a network to a picklefile, if it doesn't exist already or overwrite_existing is True"""
    force_pickle_write = False
    if not os.path.exists(output_picklefile):
        force_pickle_write = True
    elif (overwrite_existing is True) and (os.stat(output_picklefile).st_size > 0):
        force_pickle_write = True
    if force_pickle_write is True:
        logging.info("wrote pickle to " + output_picklefile)
        current_network.write_pickle(output_picklefile)
    else:
        logging.info("picklefile {0} not written, because it already exists and the --overwrite option is not True".format(output_picklefile))


if __name__ == '__main__':
    options = get_options()
#    if options.windows_size != 0:
    generate_report_windows(options, options.report_type)
    options_line = "genes_file: {0}\tparams_file: {1}\twindow_size: {2}\toverlapping: {3}\tn_individuals: {4}".format(options.genes_file_path, options.params_file, options.windows_size, options.overlapping, options.n_individuals)
    logging.debug("network report generated correctly - " + options_line)
#    else:
#        generate_report_old(options)




#!/usr/bin/env python
"""
This script takes as input a VCF file, and generates a report of the Genotype Network properties from it.


"""
#import get_gene_coords
import binary_to_network
import filteredvcf_to_binary
import logging
import argparse
import os
import random
#import numpy
try:
    from GenotypeNetwork import GenotypeNetworkInitError
    import GenotypeNetwork
except ImportError:
    from src.GenotypeNetwork import GenotypeNetworkInitError as GenotypeNetworkInitError
    from src.GenotypeNetwork import GenotypeNetwork
import yaml
import re
#import gc
#import multiprocessing

def get_options():
    parser = argparse.ArgumentParser(#            description=__doc__,
            formatter_class=argparse.ArgumentDefaultsHelpFormatter,
            prog="vcf2networks")
#    config_group = argparse.OptionGroup(parser, "Configuration options. Required")
    config_group = parser.add_argument_group("Configuration options. Required")
    config_group.add_argument('-v', '-g', '--vcf', '--genotypes', dest='vcf_file', 
            help='VCF file to be analyzed. Genotypes must be sorted by position, and belong to the same chromosome. Only Single Nucleotide Polimorphisms, and diploid individuals, are supported. Required.', 
            default=False)
    config_group.add_argument('-i', '--individuals_file', '--individuals', dest='individuals_file', 
            help='File containing the classification of individuals into populations. This must be a tab-separated file, in which the first column is the individual ID, the second is the population, the third is the continental group (or other classification), and the other columns contain classification in phenotypes, one per column. This can also defined in the parameters file (--config). Required.', 
            default="")
    config_group.add_argument('-c', '--config', '--params', dest='params_file', 
            help='File containing many parameters of the analysis. In particular, the list of network properties that will be included in the output report. Required.', 
            default=False)
    config_group.add_argument('-p', '--property', '--phenotype', dest='phenotype', 
            help='Phenotype column used to define the Genotype Networks (e.g. population, disease status, eye color, etc..). When this option is defined, the output will consist of one line for the global population, plus one line for each Genotype Network built using only individuals having the same phenotype.', 
            default="continent")
    config_group.add_argument('-o', '--report_type', dest='report_type', 
#            help='Type of report to produce.  \nPossible values: "interpop_comparison", "values_by_window", "values_by_node", "individuals", and "individuals_line".',
#            choices=("values_by_window", "values_by_node", "interpop_comparison", "individuals", "individuals_line"),
            help='Type of report to produce. "values_by_window" produces one line per each network, giving the average and the maximum of each property (see the configuration file). "values_by_node" produces one line per each node in the network (this will be a very long output). "individuals_line" will print details of the individuals used in the network (this output type is useful when subsampling individuals, combined with the --random_seed option).',
            choices=("values_by_window", "values_by_node", "individuals_line", "individuals"),
            default="values_by_window")
#    parser.add_option_group(config_group)

    network_definition_group = parser.add_argument_group("Network Definition options")
    network_definition_group.add_argument('-n', '-d', '--distance', dest='distance', type=int,
            help='Distance between nodes in the network. Two nodes will be considered as neighbors if they have at least _distance_ number of differences.',
            default=1)
    network_definition_group.add_argument('-w', '--windows_size', '--windows', dest='windows_size', type=int,
            help='Split the region in non-overlapping windows of a given size. If not provided, only a single value for all the genotypes in the VCF file will be provided.', 
            default=0)
    network_definition_group.add_argument('-l', '--overlapping', '--overlapping_windows', dest='overlapping', action='store_true',
            help='Should sliding windows be overlapping? If True, each window will start from the second base of the previous window.',
            default=False)
    network_definition_group.add_argument('-s', '--sample', '--n_haplotypes', '--n_datapoints', dest='n_individuals', type=int,
            help='Sample only this number of individuals. The output will contain an additional line for the global population after subsampling (global_sub), and all the networks will be built using only this fixed number of individuals chosen at random. See also the --replicates and --random_seed options. Also, note that n_individuals = n_haplotypes/2.',
             # NOTE: this var is called n_individuals, although it actually measures n_haplotypes
            default=0)
    network_definition_group.add_argument('-r', '--replicates', '--number_subsampling', dest='n_subsampling', type=int,
            help='How many times to resample individuals (when the --sample option is given).', 
            default=1)
    network_definition_group.add_argument('--random_seed', dest='random_seed', type=int,
            help='Random seed, for sampling individuals. If 0, a random number is used.',
            default=0)
#    parser.add_option_group(network_definition_group)

#    performances_group = argparse.OptionGroup(parser, "Options to save/read networks to files")
#    performances_group.add_argument('--hdf5', '--h5',
#            help='Use HDF5 internal storage. More memory efficient, but requires the h5py library to be installed.',
#            dest='use_hdf5', action='store_true', default=False)
#    parser.add_option_group(performances_group)
#    notimplemented_group = argparse.OptionGroup(parser, "Options not implemented yet")
#    notimplemented_group.add_argument('--parallelize', dest='parallelize', action='store_true', 
#            help='use multiple processors for generating the report',
#            default=False)
#    parser.add_option_group(notimplemented_group)
    parser.add_argument('-u', '--no-debug', dest='debug', action='store_false', 
            help="disable debugging messages, which are enabled by default", default=False)
    options = parser.parse_args()
    
#    if options.parallelize is True:
#        raise NotImplementedError("parallelization wasn't working correctly, so I removed the code for it.")
    if options.report_type in ("interpop_comparison", ):
        raise NotImplementedError("interpop comparison report not implemented yet")
    if options.debug is True:
#        logging.basicConfig(level=logging.DEBUG, filename='logs/network_report.log')
        logging.basicConfig(level=logging.DEBUG)

    if (options.vcf_file is False):
#        parser.print_help()
        parser.error('\n\n\t\tno VCF file provided (check the --vcf option).\n\nPlease use vcf2networks --help to get usage information')
    try:
        if os.stat(options.vcf_file).st_size == 0:
            parser.error("file {0} is empty".format(options.vcf_file))
    except:
        parser.print_help()
        parser.error("Can not open genotypes file {0}".format(options.vcf_file))

    return (options)

def generate_report_windows(options, binary_file, report_type='csv_report'):
    """
    Parse a file containing binary genotypes, generate a Genotype Network for each population, and write a report showing a resume of the network properties.

    +++++++++++++++++
    Parameters
    +++++++++++++++++

    - options: see the documentation for this script (--help)
    - columns included in the report file: these are defined in the params file.

    ++++++++++++++++++++++
    Example output report:
    ++++++++++++++++++++++

    There are three possible output report:

        * values_by_window report
        * values_by_node report
        * interpop_comparison report

    ------------------------
    values_by_window report 

    This report shows one line per every window/gene.
     
    ::

        gene continent distance_definition n_snps n_datapoints av_datapoints_per_node n_vertices n_edges n_components av_path_length diameter av_degree window whole_gene_nsnps
        ALG3 glob 1 30 2184 42.8235 51 50 6 3.7783 8 1.9608 8 0.0319 window_0 76
        ALG3 glob 1 30 2184 59.027 37 31 9 3.2544 7 1.6757 12 0.0517 window_1 76
        ALG3 AFR 1 30 492 13.6667 36 31 6 2.9438 6 1.7222 5 window_0 76
        ALG3 AFR 1 30 492 21.3913 23 15 8 3.1481 7 1.3043 5 window_1 76
        ALG3 ASN 1 30 572 63.5556 9 4 5 1.2 2 0.8889 2 window_0 76


    ------------------------
    values_by_node report 

    This report shows one line per every window/gene.
     
    ::
        
        genotype    degree  closeness
        01000       2       0.6
        00000       1       0.3
        01001       1       0.3 


    ------------------------
    interpop_comparison

    Not implemented yet
     
    """
    random_seed = options.random_seed
    original_random_seed = random_seed
    if random_seed is False:
        random_seed = random.random()
#    genes = get_gene_coords.read_genes_file(options.options.vcf_file)[1]
    individuals_dict = binary_to_network.parse_individuals_file(options.individuals_file)

#    params_file = yaml.load(open(options.params_file, 'r'))
#    pops = params_file['network_phenotypes']['populations']
    report_fields = []
    if report_type == 'values_by_window':
        report_fields.extend(options.params['networkproperties_report_fields'])
        report_fields.append('window')
        report_fields.append('whole_gene_nsnps')
    elif report_type == 'values_by_node':
        report_fields.extend(['genotype', 'name', 'window', 'whole_gene_nsnps'])
        report_fields.extend(options.params["allvalues_report_fields"])
    report_fields.insert(1, options.phenotype)

#    graphmlfile_basepath = options.data_basedir + '/graphml/{}_{}_distance{}.graphml'
#    picklefile_basepath = options.data_basedir + '/pickles/{}_{}_distance{}.pickle'
#    networks = {}

    header = ' '.join(report_fields) 

    print header
#    report_output = header + '\n'
    windows_size = options.windows_size
    if options.overlapping is True:
        step = 1
    else:
        step = windows_size

#    if options.n_individuals > 1:
#        pops.insert(0, "global_sub")
    for subsample_id in range(options.n_subsampling):
        random_seed += 1  #TODO: this is the code that breaks the subsampling 
        binary_to_network.print_report_and_subgraphs(binary_file, "gene", individuals_dict, 
                report_type, report_fields, options, random_seed=random_seed, phenotype=options.phenotype)
        random_seed = original_random_seed



def main():
    options = get_options()
#    if options.windows_size != 0:
#    (genotypes_lines, unphased_genotypes, snp_ids, snp_annotations ) = filteredvcf_to_binary.read_genotypes_vcf(options.vcf_file)

    # Get the name of the individuals_file
    if options.params_file is False:
        options.params = {
                'networkproperties_report_fields': ['file_prefix', 'n_snps', 'chromosome', 'n_datapoints', 'n_components', 'av_degree', 'av_path_length'],
                'individuals_file': 'data/individuals/individuals_annotations.txt'}
    else:
        options.params = yaml.load(open(options.params_file, 'r'))

#    print options.params_file
    if options.individuals_file == "":
        if options.params.has_key('individuals_annotations'):
            options.individuals_file = options.params['individuals_annotations']
        else:
            argparse.error("no individuals file (--individuals_file option) provided")
    if not os.path.isfile(options.individuals_file): argparse.error('Invalid individuals file. Please specify one with the -i option, or in the parameters file')
#    try: with open(options.vcf_file) as f: pass except IOError as e: 'Invalid VCF file file. Please specify one with the -i option, or in the parameters file'

    # Generate a temporary file containing the genotypes.
    options.use_hdf5 = False
    binary_file_contents = filteredvcf_to_binary.print_binary_file(options.vcf_file, options)
    import tempfile
    binary_file = tempfile.TemporaryFile()
    binary_file.write(binary_file_contents)
    binary_file.seek(0)
#    print genotypes_lines
#    print snp_annotations
    generate_report_windows(options, binary_file, options.report_type)

    options_line = "vcf_file: {0}\tparams_file: {1}\twindow_size: {2}\toverlapping: {3}\tn_individuals: {4}".format(options.vcf_file, options.params_file, options.windows_size, options.overlapping, options.n_individuals)
    logging.debug("network report generated correctly - " + options_line)
#    else:
#        generate_report_old(options)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
"""
import unittest
import nose
from GenotypeNetwork import GenotypeNetworkInitError

@nose.plugins.attrib.attr("nocluster")
class Simple_5SNPs_5Inds_Network(unittest.TestCase):
    def setUp(self):
        from GenotypeNetwork import GenotypeNetwork
        self.network = GenotypeNetwork(5, 'sample_phenotype')
        self.genotypes = ['00001', '01000', '00000', '01001', '01101']
        self.network.populate_from_binary_strings(self.genotypes)

    def test_summary(self):
        print "PRINT", self.network.summary()
        expected_output = "5 nodes, 5 edges, undirected\n\nNumber of components: 1\nDiameter: 3\nDensity: 0.5000\nAverage path length: 1.6000"

        output = self.network.summary()
        print output
        self.assertEqual(output, expected_output)


@nose.plugins.attrib.attr("nocluster")
def test_GoodGenotypes():
    """
    Test different types of valid inputs for GenotypeNetwork, they should all work without errors
    """
    good_genotypes = [
        (3, ['001', '000', '010'], ""),
        (1, ['1', '0', '0'], ""),
        (4, ['0001', '0010', '1010'], ""),
        (3, ['000', '000', '000'], ""),
        (5, ['00001', '11000', '01110'], ""),
        (3, ['111', '100', '010'], ""),
        ]
    for params_set in good_genotypes:
        yield check_GoodGenotypes, params_set[0], params_set[1], params_set[2]


def check_GoodGenotypes(chromosome_len, genotypes, expectedoutput):
    from GenotypeNetwork import GenotypeNetwork
    net = GenotypeNetwork(chromosome_len, genotypes=genotypes)

@nose.plugins.attrib.attr("nocluster")
def test_WrongGenotypes():
    """
    Test different types of wrong inputs for GenotypeNetwork, they should all return errors
    """
    wrong_genotypes = [
        (3, ['001', '000', '020'], ""),         # contains a "2"
        (2, ['001', '000', '010'], ""),         # wrong chromosome_len
        (3, ['001', '000', '00'], ""),          # wrong length
        (5, ['00101', '11000', '0000'], ""),    # wrong length
        (3, '001', ''),                         # genotypes is not a list
        ]

    for params_set in wrong_genotypes: 
        yield check_WrongGenotypesError, params_set[0], params_set[1], params_set[2]

@nose.tools.raises(GenotypeNetworkInitError)
def check_WrongGenotypesError(chromosome_len, genotypes, expectederrormessage):
    from GenotypeNetwork import GenotypeNetwork
    print chromosome_len, genotypes
    net = GenotypeNetwork(chromosome_len, genotypes=genotypes)
    print genotypes


if __name__ == '__main__':
    import logging
    import doctest
    logging.basicConfig(level=logging.DEBUG)
    doctest.testmod()





#!/usr/bin/env python
"""
"""
import unittest
import nose
import GenotypeNetwork
import binary_to_network

testfiles = [
    "test/test_data/MOGS.binary"
   ]

pickledir = 'test/test_data/pickles/'

class MOGS(unittest.TestCase):
    gene_name = 'MOGS'
    binary_genotypes_file = 'test/test_data/binary_genotypes/{0}.binary'.format(gene_name)
    graphml_file = 'test/test_data/graphml/MOGS.graphml'
    individuals_file = 'test/test_data/phase1_integrated_calls.20101123.ALL.panel.individuals'

#    def __init__(self, gene_name):
#        self.gene_name = gene_name
#        self.binary_genotypes_file = 'test/test_data/{}.binary'.format(self.gene_name)
#
    def setUp(self): # TODO: see if you can use setUpAll
        self.individuals_dict = binary_to_network.parse_individuals_file(self.individuals_file, pickle_from_file_if_possible=False) #TODO: de-activate pickling of this variable

        self.all_genotypes, self.individuals, self.individuals_dict, self.gene_name = binary_to_network.parse_binary_file(
                    self.binary_genotypes_file, self.individuals_dict)
        self.binary_network = binary_to_network.generate_network(self.all_genotypes, self.individuals, self.individuals_dict, self.gene_name) 
        
        self.binary_network.write_graphml(self.graphml_file)
        self.graphml_network = GenotypeNetwork.network_from_file(self.graphml_file)
        print self.graphml_network

    def test_NumberVertices(self):
        assert(self.binary_network.vcount() == self.graphml_network.vcount())

    def test_NumberEdges(self):
        assert(self.binary_network.ecount() == self.graphml_network.ecount())

    def test_VertexAttributes(self):
        print(self.binary_network.vertex_attributes(), self.graphml_network.vertex_attributes())
        print self.graphml_network.vs['id']
#        assert False
        assert([(att in self.graphml_network.vertex_attributes()) for att in  self.binary_network.vertex_attributes()])

    def test_NetworkAttributes(self):
        print(self.binary_network.attributes(), self.graphml_network.attributes())
        print self.graphml_network.vs['id']
#        assert False
        assert([(att in self.graphml_network.attributes()) for att in  self.binary_network.attributes()])

    def test_VertexGenotypes(self):
        assert(self.binary_network.vs['genotype'] == self.graphml_network.vs['genotype'])

    def test_Str(self):
        print(self.binary_network.__str__(), self.graphml_network.__str__())
        assert(self.binary_network.__str__() == self.graphml_network.__str__())

    def test_CsvReport(self):
        print(self.binary_network.csv_report(), self.graphml_network.csv_report())
        assert(self.binary_network.csv_report() == self.graphml_network.csv_report())

class ALG3(MOGS):
    """
    """
    gene_name = 'ALG3'
    binary_genotypes_file = 'test/test_data/binary_genotypes/{0}.binary'.format(gene_name)

#def test_ALG13
#    The ALG13 gene contains some 2, so it should return errors

@nose.plugins.attrib.attr('slow')
class RPN2(MOGS):
    gene_name = 'RPN2'
    binary_genotypes_file = 'test/test_data/binary_genotypes/{0}.binary'.format(gene_name)


if __name__ == '__main__':
    import logging
    import doctest
    logging.basicConfig(level=logging.DEBUG)
    doctest.testmod()


